using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using KS.Gantt;

namespace CustomDrawSample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            if (!DesignMode)
                this.Font = SystemFonts.IconTitleFont;

            SetGanttDefaults();
            AddSampleData();
        }

        void SetGanttDefaults()
        {
            this.gantt1.DiagramType = KS.Gantt.Gantt.DiagramTypes.ResourcePlan;

            gantt1.RowHeight = 26;
            gantt1.BarHeight = 16;

            // Disable random bar colors.            
            gantt1.RandomBarColors = false;

            // All bars will get the DefaultBarFormatStyle
            gantt1.DefaultBarFormatStyle.BackgroundStyle.Color = Color.Chartreuse;
            
            // Name for new tasks
            gantt1.NewTaskName = "New Tour";

            // Disable Dependencies
            gantt1.AllowDependencies = false;

            // Remove the 'Role' Column from the DataGrid
            // ganttDataGrid1.ResourceTableColumns.Remove(ganttDataGrid1.ResourceTableColumns["Role"]);

            gantt1.AfterDrawHeader += new Gantt.AfterDrawHeaderEventHandler(gantt1_AfterDrawHeader);

            gantt1.BarStyle = Gantt.BarStyles.Rounded;

            // Allow user to Zoom-In up to a minute
            gantt1.MaxZoomLevel = Gantt.ZoomLevels.Minute;

            // Our custom Tooltips
            gantt1.BarTooltips = Gantt.TooltipDisplayOptions.Custom;
            gantt1.ItemTooltip += new Gantt.ItemTooltipEventHandler(gantt1_ItemTooltip);

            // Custom Item Text
            gantt1.ItemTextDisplay = Gantt.ItemTextDisplayModes.Custom;
            gantt1.CustomItemTextDisplay += new Gantt.ItemTextDisplayEventHandler(gantt1_CustomItemTextDisplay);
        }                

        void AddSampleData()
        {
            gantt1.SuspendItemLayout();

            // *** Add some Resources ***
            ResourceItem resource;

            for (int i = 1; i < 12; i++)
            {
                resource = gantt1.AddResource("Car " + i.ToString());
                resource.RowPosition = i;
            }


            // *** Add some Tasks ***
            TaskItem task;

            task = new TaskItem(gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(-10);
            task.Duration = TimeSpan.FromDays(6);
            task.AddResource(gantt1.Resources[2]);
            task.Text = "Way 1";
            task.SetProperty("Tour", "Tour 1");
            gantt1.AddTask(task);

            task = new TaskItem(gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(0);
            task.Duration = TimeSpan.FromDays(8);
            task.AddResource(gantt1.Resources[2]);
            task.Text = "Way 2";
            task.SetProperty("Tour", "Tour 1");
            gantt1.AddTask(task);

            task = new TaskItem(gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(14);
            task.Duration = TimeSpan.FromDays(10);
            task.AddResource(gantt1.Resources[2]);
            task.Text = "Way 3";
            task.SetProperty("Tour", "Tour 1");
            gantt1.AddTask(task);

            // Another Tour
            task = new TaskItem(gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(36);
            task.Duration = TimeSpan.FromDays(4);
            task.AddResource(gantt1.Resources[2]);
            task.Text = "Way 4";
            task.SetProperty("Tour", "Tour 2");            
            gantt1.AddTask(task);
            task.FormatStyle.BackgroundStyle.Color = Color.Coral;

            task = new TaskItem(gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(44);
            task.Duration = TimeSpan.FromDays(7);
            task.AddResource(gantt1.Resources[2]);
            task.Text = "Way 5";
            task.SetProperty("Tour", "Tour 2");
            gantt1.AddTask(task);
            task.FormatStyle.BackgroundStyle.Color = Color.Coral;

            // another Car, another Tour
            task = new TaskItem(gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(2);
            task.Duration = TimeSpan.FromDays(7);
            task.AddResource(gantt1.Resources[8]);
            task.Text = "Way 1";
            task.SetProperty("Tour", "Tour 3");
            gantt1.AddTask(task);
            task.FormatStyle.BackgroundStyle.Color = Color.BlueViolet;

            task = new TaskItem(gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(15);
            task.Duration = TimeSpan.FromDays(12);
            task.AddResource(gantt1.Resources[8]);
            task.Text = "Way 2";
            task.SetProperty("Tour", "Tour 3");
            gantt1.AddTask(task);
            task.FormatStyle.BackgroundStyle.Color = Color.BlueViolet;
            

            // Perform the LayoutLogic
            gantt1.AutoMoveItems();

            gantt1.ResumeItemLayout();
        }

        void DrawRoundedRectangle()
        {

        }

        
        private class TaskItemComparerByCustomField : IComparer<TaskItem>
        {
            // Sort
            // 1. RowIndex
            // 2. Tour
            // 3. DateTime

            public int Compare(TaskItem item1, TaskItem item2)
            {
                int retVal = item1.RowIndex.CompareTo(item2.RowIndex);
                if (retVal == 0)
                {
                    retVal = ((string)item1.GetProperty("Tour")).CompareTo((string)item2.GetProperty("Tour"));
                    if (retVal == 0)
                    {
                        retVal = item1.CompareTo(item2);
                    }
                }

                return retVal;
            }
        }


        void gantt1_AfterDrawHeader(object sender, GanttDrawEventArgs e)
        {
            // here we do our custom drawing
            // We group by property 'Tour' and draw a custom background rectangle for each tour

            if (gantt1.Tasks.Count == 0)
                return;

            gantt1.Tasks.Sort(new TaskItemComparerByCustomField());            

            KS.Gantt.BoxFormatStyle bfstyle = new BoxFormatStyle();
            bfstyle.BackgroundStyle.GlossBlendStyle = GradientBlendStyle.BlendBarEx;
            bfstyle.BackgroundStyle.Color = Color.FromArgb(128, Color.BurlyWood);
            bfstyle.BackgroundStyle.GlossColor = Color.FromArgb(128, Color.Yellow);
            bfstyle.BorderStyle.Color = Color.FromArgb(255, Color.Gold);

            // initial values for loop
            DateTime minDate = gantt1.Tasks[0].StartDate;
            DateTime maxDate = gantt1.Tasks[0].EndDate;

            string LastTour = (string)gantt1.Tasks[0].GetProperty("Tour");
            int LastRowIndex = gantt1.Tasks[0].RowIndex;            

            for (int i = 0; i < gantt1.Tasks.Count; i++)
            {
                TaskItem task = gantt1.Tasks[i];
                string CurrentTour = (string)task.GetProperty("Tour");

                if (LastTour != CurrentTour || task.RowIndex != LastRowIndex || i == gantt1.Tasks.Count - 1)
                {
                    if (i == gantt1.Tasks.Count - 1)
                        maxDate = maxDate = task.EndDate;

                    Rectangle r = gantt1.RowBounds(LastRowIndex);
                    int x1 = gantt1.Date2x(minDate);
                    int x2 = gantt1.Date2x(maxDate);
                    Rectangle r2 = new Rectangle(x1, r.Y, x2 - x1, r.Height);
                    r2.Inflate(6, 0);

                    // only paint if on screen
                    if (r2.IntersectsWith(gantt1.ClientRectangle))
                    {
                        bfstyle.DrawBox(e.Gfx, r2, Gantt.BarStyles.Rounded);
                    }

                    minDate = task.StartDate;
                    maxDate = task.EndDate;

                    LastTour = CurrentTour;
                    LastRowIndex = task.RowIndex;                    
                }
                else
                {
                    maxDate = task.EndDate;
                }
            }

            // Important!! Always dispose FormatStyles
            bfstyle.Dispose();
        }

        // Custom Tooltips
        string gantt1_ItemTooltip(object sender, GanttItemEventArgs e)
        {
            if (e.Item.ItemType == GanttItemType.TaskItem)
            {
                return (string)e.Item.GetProperty("Tour") + " - " + e.Item.Text + "\r\n"
                    + e.Item.DateRangeForDisplay + "\r\n"
                    + e.Item.DurationForDisplay;
            }

            return e.Item.Text;
        }

        // Custom ItemText
        void gantt1_CustomItemTextDisplay(object sender, GanttItemTextEventArgs e)
        {
            if (e.Item.ItemType == GanttItemType.TaskItem)
            {
                e.Text = (string)e.Item.GetProperty("Tour") + " - " + e.Item.Text;
            }
        }
    }
}