using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using KS.Gantt;

namespace GanttDemo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            if (!DesignMode)
                this.Font = SystemFonts.IconTitleFont;

            // Show the property grid for both controls
            gantt1.Enter += new EventHandler(gantt1_Enter);
            ganttDataGrid1.Enter += new EventHandler(ganttDataGrid1_Enter);

            // Some other events
            gantt1.BeforeDelete += new KS.Gantt.Gantt.BeforeDeleteEventHandler(gantt1_BeforeDelete);
            gantt1.ItemDoubleClick += new Gantt.ItemDoubleClickEventHandler(gantt1_ItemDoubleClick);

            gantt1.MovingItem += new Gantt.MovingItemEventHandler(gantt1_MovingItem);
            gantt1.ItemsMoved += new EventHandler(gantt1_ItemsMoved);
            gantt1.ItemsResized += new EventHandler(gantt1_ItemsResized);
            gantt1.ItemHover += new Gantt.ItemHoverEventHandler(gantt1_ItemHover);
            
            gantt1.GranularityChanged += new EventHandler(gantt1_GranularityChanged);
        }        
        

        private void Form1_Load(object sender, EventArgs e)
        {            
            this.gantt1.ShowIcons = false;
            this.gantt1.SelectionMode = Gantt.SelectionModes.MultipleSelection;

            this.gantt1.StartDate = DateTime.Now.AddMonths(0);
            this.gantt1.EndDate = DateTime.Now.AddMonths(2);

            GanttDataColumn col = this.ganttDataGrid1.TaskColumns.Add("MyColumn", "My Column");
            col.EditType = GanttEditTypes.DropDownList;
            col.AddValueListItem("", null);
            col.AddValueListItem("Banana", 1);
            col.AddValueListItem("Tomato", 2);
            col.AddValueListItem("Orange", 3);


            gantt1.SuspendItemLayout();

            ResourceItem resource1 = this.gantt1.AddResource("Detlef Kroll", "Developer", 100.0);
            ResourceItem resource2 = this.gantt1.AddResource("Thomas Thompson", "Project-Manager", 140.0);
            ResourceItem resource3 = this.gantt1.AddResource("David Stevens", "Marketing", 75.0);
            ResourceItem resource4 = this.gantt1.AddResource("Richard Reed", "Office", 48.0);
            
            // Now add some tasks and groups and relations
            
            TaskItem bar1;
            bar1 = this.gantt1.AddTask(new DateTime(2015, 1, 2), new DateTime(2015, 1, 20), "TaskItem 1");
            bar1.PercentDone = 0.82;
            bar1.AddResource(resource1, 0.5);
            bar1.AddResource(resource2, 0.5);
            bar1.AddResource(resource3, 0.7);            
            bar1.Status = TaskItem.TaskStates.Running;
            bar1.Priority = TaskItem.TaskPriorities.High;
            bar1.SetProperty("MyColumn", "Value 1");


            TaskItem bar2;
            bar2 = this.gantt1.AddTask(new DateTime(2015, 1, 22), new DateTime(2015, 2, 27), "TaskItem 2");
            bar2.PercentDone = 0.63;            
            bar2.AddResource(resource1, 0.5);
            bar2.AddResource(resource2, 0.7);
            bar2.AddDependence(bar1, 5);
            bar2.Status = TaskItem.TaskStates.Running;
            bar2.Priority = TaskItem.TaskPriorities.Lower;
            bar2.SetProperty("MyColumn", "Value 2");

            TaskItem bar3;
            bar3 = this.gantt1.AddTask(new DateTime(2015, 1, 12), new DateTime(2015, 2, 20), "TaskItem 3");
            bar3.PercentDone = 0.33;            
            bar3.AddResource(resource1, 0.5);
            bar3.AddResource(resource2, 0.3);
            bar3.AddResource(resource3, 0.8);
            bar3.AddDependence(bar2);
            bar3.Status = TaskItem.TaskStates.Paused;
            bar3.Priority = TaskItem.TaskPriorities.Higher;

            TaskItem bar4;
            bar4 = this.gantt1.AddTask(new DateTime(2015, 3, 5), new DateTime(2015, 3, 20), "TaskItem 4");
            bar4.PercentDone = 0.3;            
            bar4.AddResource(resource1, 0.5);
            bar4.AddResource(resource3, 0.9);
            bar4.AddDependence(bar3);
            bar4.Status = TaskItem.TaskStates.Waiting;
            bar4.Priority = TaskItem.TaskPriorities.High;

            TaskItem bar4a;
            bar4a = this.gantt1.AddTask(new DateTime(2015, 3, 21), new DateTime(2015, 3, 21), "Milestone 1");
            bar4a.IsMilestone = true;            
            bar4a.AddDependence(bar4);
        

            TaskItem bar5;
            bar5 = this.gantt1.AddTask(new DateTime(2015, 4, 5), new DateTime(2015, 4, 12), "TaskItem 5");
            bar5.PercentDone = 0.3;            
            bar5.AddResource(resource1, 0.5);
            bar5.Status = TaskItem.TaskStates.Waiting;
            bar5.Priority = TaskItem.TaskPriorities.Normal;

            TaskItem bar6;
            bar6 = this.gantt1.AddTask(new DateTime(2015, 4, 15), new DateTime(2015, 4, 22), "TaskItem 6");
            bar6.PercentDone = 0.3;            
            bar6.AddResource(resource1, 0.5);
            bar6.AddResource(resource4, 0.6);
            bar6.AddDependence(bar5);
            bar6.Status = TaskItem.TaskStates.Paused;
            bar6.Priority = TaskItem.TaskPriorities.Lower;

            TaskItem bar7;
            bar7 = this.gantt1.AddTask(new DateTime(2015, 4, 22), new DateTime(2015, 5, 3), "TaskItem 7");
            bar7.PercentDone = 0.3;            
            bar7.AddResource(resource1, 0.5);
            bar7.AddResource(resource4, 0.7);
            bar7.AddDependence(bar6);
            bar7.Status = TaskItem.TaskStates.Steady;
            bar7.Priority = TaskItem.TaskPriorities.Higher;

            TaskItem bar8;
            bar8 = this.gantt1.AddTask(new DateTime(2015, 5, 5), new DateTime(2015, 5, 16), "TaskItem 8");
            bar8.PercentDone = 0.3;            
            bar8.AddResource(resource1, 0.5);
            bar8.AddResource(resource4, 0.3);
            bar8.AddResource(resource3, 0.6);
            bar8.AddDependence(bar7);
            bar8.Status = TaskItem.TaskStates.Steady;
            bar8.Priority = TaskItem.TaskPriorities.Normal;

            GroupItem gi1 = gantt1.AddGroup("Group 1");
            gi1.AddChild(gantt1.Tasks[0]);
            gi1.AddChild(gantt1.Tasks[1]);
            gi1.AddChild(gantt1.Tasks[2]);
            gi1.AddChild(gantt1.Tasks[3]);
            gi1.AddChild(gantt1.Tasks[4]);

            GroupItem gi2 = gantt1.AddGroup("Group 2");
            gi2.AddChild(gantt1.Tasks[5]);
            gi2.AddChild(gantt1.Tasks[6]);
            gi2.AddChild(gantt1.Tasks[7]);
            gi2.AddChild(gantt1.Tasks[8]);

            GroupItem gi0 = gantt1.AddGroup("Group 0");
            gi0.AddChild(gi1);
            gi0.AddChild(gi2);


            TaskItem bar9;
            bar9 = this.gantt1.AddTask(new DateTime(2015, 5, 5), new DateTime(2015, 5, 16), "TaskItem 9");
            bar9.PercentDone = 0.3;            
            bar9.AddResource(resource1, 0.5);
            bar9.AddResource(resource4, 0.3);
            bar9.AddResource(resource3, 0.6);
            bar9.Status = TaskItem.TaskStates.Done;
            bar9.Priority = TaskItem.TaskPriorities.High;

            TaskItem bar10;
            bar10 = this.gantt1.AddTask(new DateTime(2015, 4, 1), new DateTime(2015, 4, 16), "TaskItem 10");
            bar10.PercentDone = 0.3;            
            bar10.AddResource(resource1, 0.8);
            bar10.AddResource(resource4, 0.2);
            bar10.Status = TaskItem.TaskStates.Steady;
            bar10.Priority = TaskItem.TaskPriorities.Normal;

            GroupItem gi3 = gantt1.AddGroup("Group 3");
            gi3.AddChild(bar9);

            gi1.AddChild(gi3);

            GroupItem gi4 = gantt1.AddGroup("Group 4");
            gi4.AddChild(bar9);
            gi1.AddChild(gi4);
            TaskItem bar11;
            bar11 = this.gantt1.AddTask(new DateTime(2015, 5, 1), new DateTime(2015, 5, 16), "TaskItem 11");
            bar11.PercentDone = 0.3;            
            bar11.AddResource(resource1, 0.8);
            bar11.AddResource(resource4, 0.2);
            bar11.Status = TaskItem.TaskStates.Steady;
            bar11.Priority = TaskItem.TaskPriorities.Normal;

            TaskItem bar12;
            bar12 = this.gantt1.AddTask(new DateTime(2015, 5, 17), new DateTime(2015, 5, 30), "TaskItem 12");
            bar12.PercentDone = 0.3;            
            bar12.AddResource(resource1, 0.8);
            bar12.AddResource(resource4, 0.2);
            bar12.Status = TaskItem.TaskStates.Steady;
            bar12.Priority = TaskItem.TaskPriorities.Normal;

            TaskItem bar13;
            bar13 = this.gantt1.AddTask(new DateTime(2015, 6, 1), new DateTime(2015, 6, 16), "TaskItem 13");
            bar13.PercentDone = 0.3;            
            bar13.AddResource(resource1, 0.8);
            bar13.AddResource(resource4, 0.2);
            bar13.Status = TaskItem.TaskStates.Steady;
            bar13.Priority = TaskItem.TaskPriorities.Normal;

            // Circular Dependency
            bar12.AddDependence(bar11);
            bar13.AddDependence(bar12);
            bar11.AddDependence(bar13);

            //gantt1.EnsureItemVisible(bar11);

            gantt1.AutoMoveItems();    

            gantt1.ZoomToFit();            

            gantt1.ResumeItemLayout();
            gantt1.Focus();

            // Start up Maximized
            this.WindowState = FormWindowState.Maximized;
            splitContainer1.SplitterDistance = 320;
            splitContainer2.SplitterDistance = 460;
        }        

        void gantt1_Enter(object sender, EventArgs e)
        {
            propertyGrid1.SelectedObject = gantt1;
        }

        void ganttDataGrid1_Enter(object sender, EventArgs e)
        {
            propertyGrid1.SelectedObject = ganttDataGrid1;
        }

        void gantt1_BeforeDelete(object sender, KS.Gantt.GanttCancelEventArgs e)
        {
            if (MessageBox.Show(this, "Do you really want to delete " + gantt1.SelectedItems.Count.ToString() + " selected items ?", "Gantt Demo", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question) != DialogResult.Yes)
                e.Cancel = true;
        }
        
        void gantt1_ItemDoubleClick(object sender, GanttItemMouseEventArgs e)
        {
            gantt1.EditItem(e.Item);            
        }

        void gantt1_MovingItem(object sender, GanttItemEventArgs e)
        {
            ShowItemStatus(e.Item);
        }

        void gantt1_ItemsMoved(object sender, EventArgs e)
        {
            ShowItemStatus(null);
        }

        void gantt1_ItemsResized(object sender, EventArgs e)
        {
            ShowItemStatus(null);
        }

        void gantt1_ItemHover(object sender, GanttItemEventArgs e)
        {
            ShowItemStatus(e.Item);
        }

        void gantt1_GranularityChanged(object sender, EventArgs e)
        {
            if (gantt1.Granularity == Granularities.Day)
            {
                toolStripStatusLabelStartDate.Width = 80;
                toolStripStatusLabelEndDate.Width = 80;
                toolStripStatusLabelDuration.Width = 64;
            }
            else
            {
                toolStripStatusLabelStartDate.Width = 110;
                toolStripStatusLabelEndDate.Width = 110;
                toolStripStatusLabelDuration.Width = 70;
            }

            ShowItemStatus(null);
        }

        private void ShowItemStatus(GanttItem item)
        {
            if (item == null)
            {
                toolStripStatusLabelTaskName.Text = "";
                toolStripStatusLabelStartDate.Text = "";
                toolStripStatusLabelEndDate.Text = "";
                toolStripStatusLabelDuration.Text = "";
            }
            else
            {
                string strDateFormat;
                string strDurationFormat;

                if (gantt1.Granularity == Granularities.Day)
                {
                    strDateFormat = "d";
                    strDurationFormat = "n0";
                }
                else
                {
                    strDateFormat = "g";
                    if (gantt1.Granularity == Granularities.HalfDay)
                        strDurationFormat = "n1";
                    else
                        strDurationFormat = "n2";
                }

                toolStripStatusLabelTaskName.Text = item.Text;
                toolStripStatusLabelStartDate.Text = item.StartDate.ToString(strDateFormat);
                toolStripStatusLabelEndDate.Text = item.EndDate.ToString(strDateFormat);
                toolStripStatusLabelDuration.Text = item.Duration.TotalDays.ToString(strDurationFormat) + " days";                
            }
        }
    }
}