using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using KS.Gantt;

namespace HelloWorld
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            if (!DesignMode)
                this.Font = SystemFonts.IconTitleFont;

            // *** First steps we did..
            // Add a reference to the KS-Gantt Control for WinForms
            // Added using KS.Gantt above

            // We added a SplitContainer to the form
            // Dropped a GanttDataGrid into the left SplitterPanel
            // Dropped a Gantt into the right SplitterPanel

            // Bind the GanttDataGrid to the Gantt-Control            
            // ganttDataGrid1.GanttControl = gantt1; // already done in the designer

            // *** No we enable editing with the standard dialogs..
            gantt1.ItemDoubleClick += new KS.Gantt.Gantt.ItemDoubleClickEventHandler(gantt1_ItemDoubleClick);

            // *** Set some defaults
            gantt1.DefaultTaskType = TaskItem.TaskTypes.FixedUnits;
            gantt1.ItemTextDisplay = Gantt.ItemTextDisplayModes.TextAndResources;
            gantt1.BarTooltips = Gantt.TooltipDisplayOptions.TextAndDateAndDescription;
            gantt1.RandomBarColors = true;  // this is the default

            gantt1.RowHeight = 24;  // this is the default
            gantt1.BarHeight = 12;  // this is the default

            // Set the Granularity
            gantt1.Granularity = Granularities.QuarterHour;
            gantt1.MaxZoomLevel = Gantt.ZoomLevels.Minute;
            gantt1.PathDimming = true;
            
            // You want a readonly control ?
            //gantt1.AllowEdit = false;
            //ganttDataGrid1.AllowEdit = false;
            //ganttDataGrid1.SelectionMode = GanttDataGrid.GanttDataGridSelectionModes.Row;            

            gantt1.SuspendItemLayout();

            // *** and add a new task
            TaskItem task1 = gantt1.AddTask(DateTime.Now, DateTime.Now.AddDays(5), "Hello World");
            TaskItem task2 = gantt1.AddTask(DateTime.Now, DateTime.Now.AddDays(5), "Task 2");

            // Add a Dependency
            TaskRelation rel = task2.AddDependence(task1);
            //rel.DelayDays = 0;  // can be negative !
            //rel.TaskRelationType = TaskRelationTypes.EndStart;            
            //rel.DelayDurationType = TaskItem.DurationTypes.FullWorkingDays;
            //rel.TaskRelationMode = TaskRelationModes.Hard;            

            // Add a group and assign the tasks
            GroupItem group = gantt1.AddGroup("Group 1");
            group.AddChild(task1);
            group.AddChild(task2);            

            gantt1.ResumeItemLayout();
        }

        void gantt1_ItemDoubleClick(object sender, KS.Gantt.GanttItemMouseEventArgs e)
        {
            gantt1.EditItem(e.Item);
        }

        // ****************************** Thechniques ***********************************
        // The following functions are never called. They just show some common techniques


        void SetModes()
        {
            // Set Diagram-Type ResourcePlan
            // In ResourcePlan, different Tasks can be positioned on the same row.
            // Every task has exactly one Resource assigned
            // The Resource will be assigned automatically when moving task
            gantt1.DiagramType = Gantt.DiagramTypes.ResourcePlan;

            // Set Diagram-Type Gantt
            gantt1.DiagramType = Gantt.DiagramTypes.Gantt;

            // In a Gantt-Diagram, we have two View:

            // design the project
            gantt1.ViewMode = Gantt.ViewModes.Tasks;

            // see resource-allocations
            // you must add some resources to the tasks, to see something here
            gantt1.ViewMode = Gantt.ViewModes.Resources;
        }

        void SetTaskTypes()
        {
            // a task can have 4 different types.
            // please see
            // http://www.kroll-software.de/projectplanner/de/the_heart_of_pm.asp
            
            TaskItem task = gantt1.Tasks[0];
            task.TaskType = TaskItem.TaskTypes.StaticTask;
            task.Recalculate(true);
        }

        void ShortCutsOfTheControl()
        {
            // please see
            // http://www.kroll-software.de/projectplanner/en/start.asp
        }

        void BulkUpdating()
        {
            // When doing many updates, disable the painting, layout and calculations
            gantt1.SuspendItemLayout();

            // Do many updates and changes on the control here

            gantt1.ResumeItemLayout();

            // Suspend/Resume rises a counter internally and can be used nested
            // Simply ensure, that both are always called in pair..

            // the same can be done with tasks to 
            // suspend the INotifyPropertyChanged and speed things up
            TaskItem task = gantt1.Tasks[0];
            task.SuspendEvents();
            // set many task properties here..
            task.ResumeEvents();
        }

        void FindingItems()
        {
            // Item at RowIndex
            int rowindex = 0;   // first row is zero
            rowindex = gantt1.MaxRowIndex;

            // in gantt1.DiagramType == Gantt.DiagramTypes.Gantt
            // this returns a TaskItem or GroupItem or null

            // in gantt1.DiagramType == Gantt.DiagramTypes.ResourcePlan
            // this returns a ResourceItem (of the row) or GroupItem or null
            GanttItem item1 = gantt1.ItemByRowIndex(rowindex);

            // Find Item by X, Y, typically Mouse coordinates
            int X = 100;
            int Y = 200;
            GanttItem item2 = gantt1.FindItem(X, Y);

            // Get date from coordinates and vice-versa
            DateTime date = gantt1.x2Date(100);
            int x = gantt1.Date2x(DateTime.Now);
        }

        void DataModel()
        {            
            // The main collections of the Gantt-Control
            TaskItemCollection Tasks = gantt1.Tasks;
            GroupItemCollection Groups = gantt1.Groups;
            ResourceItemCollection Resources = gantt1.Resources;
            SkillItemCollection Skills = gantt1.Skills;
            KS.Gantt.Calendars.CalendarCollection Calendars = gantt1.CalendarList;

            // All Collections Inherit from GanttItemCollection
            // which inherits from BaseItemCollection
            // which inherits from SortableBindingList

            TaskItem task = Tasks[0];
            GroupItem group = Groups[0];
            ResourceItem resource = Resources[0];
            // All inherit from GanttItem

            SkillItem skill = Skills[0];            
            KS.Gantt.Calendars.Calendar calendar = Calendars[0];
            // these inherits from BaseItemText

            // Class Hierarchie:
            // BaseItem => BaseItemText => BaseItemTextProperties => GanttItem => 
            // => TaskItem
            // => GroupItem => ResourceItem
            
            
            // The collections my be sorted within the control.
            // You should never depend on Item-Indices in the collections.
            // Always remember and access Items by their Key
            TaskItem t = Tasks.FindItemByKey(task.Key);
            // This returns null, if the Key was not found
            // or simply
            t = Tasks[task.Key];
            // This throws an error, if the Key was not found

            // Never change a Key, when the Item is part of a collection.
            // If really needed, use
            gantt1.Tasks.ChangeItemKey(task, "NewKey");
            // All Keys must be unique
            // Keys are always automatically created when creating a new BaseItem (Task, Group, ..)

            // If the object contains a method to add another object, always use this method
            // Don't use
            // gantt1.Tasks.Add(..
            // instead use
            TaskItem t2 = gantt1.AddTask(DateTime.Now, DateTime.Now.AddDays(5), "Hello");
            ResourceItem r2 = gantt1.AddResource("New Resource");
            t2.AddResource(r2);            
            
            // Item Selection
            GanttItemCollection selecteditems = gantt1.SelectedItems;
            GanttItem lastselected = gantt1.LastSelectedItem;
            GanttItem lasthover = gantt1.LastHoverItem;
        }


        void LayoutLogic()
        {
            // After any modifications to the data (tasks, groups, ..)
            // the Gantt-Control needs to perform all layout and gantt calculations.
            // Most of the time this is done automatically.
            
            // These calculations need plenty of time
            // so ensure, that they are not triggered to often
            // always use
            gantt1.SuspendItemLayout();
            gantt1.ResumeItemLayout();
            // when doing some bulky updates.
            // When the layout-logic was triggered during suspend, it will be called on resume.

            // In some situations you may need to trigger the LayoutLogic explicitly from your code.
            // simply call
            gantt1.AutoMoveItems();
            
            // this includes the cascade
            //gantt1.CalcParams();
            //gantt1.LayoutLogic.AutoArrangeRows();
            //gantt1.LayoutLogic.AutoMoveItems(gantt1.Tasks);

            // gantt1.LayoutLogic is a class that could be derived from LayoutLogic.LayoutLogicBase

            // The ordering of the Rows uses an IComparer<GanttItem>
            IComparer<GanttItem> MyItemComparer = new GanttItemComparer();
            gantt1.LayoutLogic.ItemComparer = MyItemComparer;            
        }

        // Default implementation of the GanttItemComparer
        //public class GanttItemComparer : IComparer<GanttItem>
        //{
        //    public int Compare(GanttItem item1, GanttItem item2)
        //    {
        //        if (item1 == item2)
        //            return 0;

        //        // Items on same level ?
        //        if (item1.ParentItem == item2.ParentItem)
        //            return item1.CompareTo(item2);

        //        // Items are on different level            
        //        if (item1.ItemType == GanttItemType.GroupItem && item2.ContainsParentDeep((GroupItem)item1))
        //            return -1;
        //        else if (item2.ItemType == GanttItemType.GroupItem && item1.ContainsParentDeep((GroupItem)item2))
        //            return 1;
        //        else
        //            return item1.DeepItem.CompareTo(item2.DeepItem);
        //    }
        //}

        void Calendars()
        {
            // The Control initializes a default calendar, 
            // that is used for painting the weekends and is the base for all items

            KS.Gantt.Calendars.Calendar defaultCalendar = gantt1.DefaultCalendar;

            // Thursday is not a working day ?
            defaultCalendar.SetWorkingDay(DayOfWeek.Thursday, false);

            // Work on tuesday from 9..12 h and from 13..14h
            defaultCalendar.WorkingDays.WorkingHours(DayOfWeek.Tuesday).Clear();
            defaultCalendar.WorkingDays.WorkingHours(DayOfWeek.Tuesday).Add(new KS.Gantt.Calendars.WorkingHour(9, 12));
            defaultCalendar.WorkingDays.WorkingHours(DayOfWeek.Tuesday).Add(new KS.Gantt.Calendars.WorkingHour(13, 14));            

            // Today is a holiday Exception?                            
            defaultCalendar.AddException(new KS.Gantt.Calendars.CalendarException(DateTime.Now, false));

            // Or special workinghours at a date ?
            KS.Gantt.Calendars.CalendarException ce = new KS.Gantt.Calendars.CalendarException(DateTime.Now.AddDays(1), true);
            ce.WorkingHours = new KS.Gantt.Calendars.WorkingHourCollection();
            ce.WorkingHours.Add(new KS.Gantt.Calendars.WorkingHour(8, 10));
            ce.WorkingHours.Add(new KS.Gantt.Calendars.WorkingHour(14, 16));
            defaultCalendar.AddException(ce);

            // Add a new Calendar to the Calendarlist
            KS.Gantt.Calendars.Calendar cal = new KS.Gantt.Calendars.Calendar();
            cal.InheritedCalendarKey = defaultCalendar.Key;
            // Calendars are 'cascading', in a same manner as Cascading-Style-Sheets (CSS) in HTML
            // cal is now totally inherited from the defaultcalendar 
            // and inherits all its workingdays and workinghours            
            // as far as if they are not otherwise defined
            cal.SetWorkingDay(DayOfWeek.Wednesday, false);
            // cal is still the same as the default calendar, except the fact, that Wednesday is no longer a WorkingDay

            // Every ResourceItem can have it's own calendar
            gantt1.AddResource("Resource 1");
            ResourceItem resource = gantt1.Resources[0];

            // Calendars are always referenced and assigned by their Key!
            resource.CalendarKey = cal.Key;
            cal.IsResourceCalendar = true;

            // Calendars can be Combined!
            resource.TaskCalendarCombinationMode = KS.Gantt.Calendars.CalendarCombinationModes.Intersection;
            resource.ProjectCalendarCombinationMode = KS.Gantt.Calendars.CalendarCombinationModes.Union;

            // TaskItems don't have their own calendar, but a calendar can be assigned
            gantt1.Tasks[0].CalendarKey = cal.Key;
            gantt1.Tasks[0].ProjectCalendarCombinationMode = KS.Gantt.Calendars.CalendarCombinationModes.Intersection;

            KS.Gantt.Calendars.CombinedCalendar cc = new KS.Gantt.Calendars.CombinedCalendar(defaultCalendar, cal, KS.Gantt.Calendars.CalendarCombinationModes.Union);
            // all settings for such a CombinedCalendar are readonly
            // but Queries such as IsWorkingDay or IsWorkingHour can be made

            // You can do some calculations on the calendars
            bool b = cal.IsWorkingDay(DateTime.Now);
            b = cal.IsWorkingHour(DateTime.Now.Date.AddHours(9));
            TimeSpan ts = cal.WorkingHoursInRange(DateTime.Now.Date.AddHours(8), DateTime.Now.Date.AddDays(3).AddHours(5));
            double d = cal.TotalWorkingHoursAtDate(DateTime.Now);
            //cal.ElapsedWorkingHoursAtDate()
            //cal.RemainingWorkingHoursAtDate()
            //cal.AddDurationToDate()
            //cal.SubstractDurationFromDate()
            // ..
        }
        
        void GeneticSolver()
        {
            gantt1.ShowSolveDialog();
        }

        void Questions()
        {
            // This sample only introduces into some basics of the control
            //
            // If you have any more questions about advanced features and techniques, 
            // please contact
            // support@kroll-software.ch
            //
            // We will collect all requests into a new FAQ-Sample
        }

    }
}