﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace KS.Common
{
    public class DisposableObject : IDisposable
    {
        protected readonly object m_SyncObject = new object();
        public object SyncObject
        {
            get
            {
                return m_SyncObject;
            }
        }

        protected bool m_Disposed = false;
        protected bool m_IsDisposing = false;

        public bool IsDisposed
        {
            get
            {
                return m_Disposed || m_IsDisposing;
            }
        }        
        
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        
        ~DisposableObject()
        {
            if (!m_Disposed)
                OnFinalizerCalled();                

            Dispose(false);
        }
        
        protected virtual void Dispose(bool disposing)
        {
            if (!this.m_Disposed && !m_IsDisposing)
            {
                m_IsDisposing = true;

                if (disposing)
                {
                    lock (m_SyncObject)
                    {
                        CleanupManagedResources();
                    }
                }

                CleanupUnmanagedResources();
                m_Disposed = true;
            }            
        }

        protected virtual void CleanupManagedResources()
        {
        }

        protected virtual void CleanupUnmanagedResources()
        {
        }

        protected virtual void OnFinalizerCalled()
        {
            //System.Diagnostics.Debug.WriteLine(String.Format("An instance of {0} was not disposed before garbage collection.", this.GetType().FullName));
        }
    }
}
