﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;

namespace Calendar
{
    //public class StudioRenderer : Office12Renderer
    public class StudioRenderer : AbstractRenderer
    {                
        public override Color HourColor
        {
            get
            {
                //return Color.FromArgb(230, 237, 247);
                return Color.FromArgb(206, 212, 223);
            }
        }

        public override Color HalfHourSeperatorColor
        {
            get
            {
                //if (m_GanttControl != null)
                //    return m_GanttControl.WeekendFormatStyle.VerticalLineStyle.Color;
                //else
                    return Color.FromArgb(194, 200, 208);
                //return Helpers.GuiStyle.GridLinesColor;
            }
        }

        public override Color HourSeperatorColor
        {
            get
            {                                
                return Color.FromArgb(194, 200, 208);
                //return Helpers.GuiStyle.GridLinesColor;
            }
        }

        public override Color WorkingHourColor
        {
            get
            {
                if (m_GanttControl != null)
                    return m_GanttControl.BackColor;
                else
                    return SystemColors.Window;
            }
        }

        public override Color BackColor
        {
            get
            {
                if (m_GanttControl != null)
                    return m_GanttControl.BackColor;
                else
                    return SystemColors.Window;
            }
        }

        public override Color HoursBackColor
        {
            get
            {
                return Color.FromArgb(206, 212, 223);
            }
        }

        public override Color SelectionColor
        {
            get
            {
                //return Color.FromArgb(241, 243, 248);
                return Color.FromArgb(241, 243, 248);
                //return SystemColors.Highlight;
            }
        }

        public override void DrawHourRange(Graphics g, Rectangle rect, bool drawBorder, bool hilight)
        {
            using (SolidBrush brush = new SolidBrush(hilight ? this.SelectionColor : this.WorkingHourColor))
            {
                g.FillRectangle(brush, rect);
            }

            if (drawBorder)
            {
                using (Pen pen = new Pen(Color.FromArgb(194, 200, 208)))
                {
                    g.DrawRectangle(pen, rect);
                }
            }
        }

        public override void DrawDayGripper(Graphics g, Rectangle rect, int gripWidth)
        {
            using (Brush m_Brush = new SolidBrush(BackColor))
                g.FillRectangle(m_Brush, rect.Left, rect.Top - 1, gripWidth, rect.Height);
            
            using (Pen m_Pen = new Pen(HalfHourSeperatorColor))
                g.DrawRectangle(m_Pen, rect.Left, rect.Top - 1, gripWidth, rect.Height);
        }

        public override void DrawHourLabel(Graphics g, Rectangle rect, int hour)
        {
            lock (m_SyncObj)
            {

                StringFormat m_Format = new StringFormat();
                m_Format.Alignment = StringAlignment.Near;
                m_Format.FormatFlags = StringFormatFlags.NoWrap | StringFormatFlags.FitBlackBox;
                m_Format.LineAlignment = StringAlignment.Center;

                //Color color = Color.FromArgb(101, 147, 207);
                Color color = Color.FromArgb(0, 0, 0);

                g.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;

                using (Pen pen = new Pen(color))
                {
                    g.DrawLine(pen, rect.Left, rect.Y, rect.Width, rect.Y);
                    
                    using (SolidBrush brush = new SolidBrush(color))
                    {
                        rect.X += 2;
                        rect.Width -= 2;

                        g.DrawString(hour.ToString("##00"), HourFont, brush, rect, m_Format);

                        Rectangle HourRectangle = rect;
                        HourRectangle.Offset(2, 1);                        
                        HourRectangle.Width = rect.Width;                        
                        
                        rect.X += (rect.Width * 2 / 3) - 3;
                        rect.Height = rect.Height / 2;
                        rect.Y += 1;
                        g.DrawString("00", MinuteFont, brush, rect, m_Format);

                        g.DrawLine(pen, rect.Left + 2, rect.Bottom - 1, rect.Width - 2, rect.Bottom - 1);                        

                        rect.Y += rect.Height;
                        g.DrawString("30", MinuteFont, brush, rect, m_Format);
                    }
                }
            }
        }

        private void DrawButton(Graphics g, Rectangle rect, Color TopColor, Color BottomColor, Color LineColor)
        {            
            Rectangle topPart = new Rectangle(rect.Left + 1, rect.Top + 1, rect.Width - 2, (int)Math.Ceiling((double)rect.Height / 3.0));
            Rectangle lowPart = new Rectangle(topPart.Left, topPart.Bottom + 1, topPart.Width + 1, rect.Height - topPart.Height - 1);
            Rectangle grafientPart = new Rectangle(topPart.Left, topPart.Bottom - (topPart.Height / 3) + 1, topPart.Width, (topPart.Height / 3) + 1);

            if (topPart.Width > 0 && topPart.Height > 0)
            {
                using (SolidBrush brush = new SolidBrush(TopColor))
                    g.FillRectangle(brush, topPart);
            }

            if (lowPart.Width > 0 && lowPart.Height > 0)
            {
                using (SolidBrush brush = new SolidBrush(BottomColor))
                    g.FillRectangle(brush, lowPart);
            }

            if (grafientPart.Width > 0 && grafientPart.Height > 0)
            {
                using (LinearGradientBrush aGB = new LinearGradientBrush(grafientPart, TopColor, BottomColor, LinearGradientMode.Vertical))
                    g.FillRectangle(aGB, grafientPart);
            }

            if (rect.Width > 0 && rect.Height > 0)
            {
                using (Pen aPen = new Pen(LineColor))
                    g.DrawRectangle(aPen, rect);
            }
        }

        private void DrawGrayButton(Graphics g, Rectangle rect)
        {
            DrawButton(g, rect, Color.FromArgb(237, 239, 243), Color.FromArgb(206, 212, 223), Color.FromArgb(133, 145, 162));
        }

        public override void DrawUserHeader(Graphics g, Rectangle rect, string UserName)
        {                        
            StringFormat m_Format = new StringFormat();
            m_Format.Alignment = StringAlignment.Center;
            m_Format.FormatFlags = StringFormatFlags.NoWrap;
            m_Format.LineAlignment = StringAlignment.Center;
            m_Format.Trimming = StringTrimming.EllipsisCharacter;

            DrawGrayButton(g, rect);

            g.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;

            if (rect.Width > 4)
            {
                rect.Offset(2, 1);
                rect.Inflate(-2, 0);                    
                g.DrawString(UserName, SystemFonts.IconTitleFont, SystemBrushes.WindowText, rect, m_Format);
            }
        }

        public override void DrawDayHeader(Graphics g, Rectangle rect, DateTime date)
        {            
            StringFormat m_Format = new StringFormat();
            m_Format.Alignment = StringAlignment.Center;
            m_Format.FormatFlags = StringFormatFlags.NoWrap;
            m_Format.LineAlignment = StringAlignment.Center;

            StringFormat m_Formatdd = new StringFormat();
            m_Formatdd.Alignment = StringAlignment.Near;
            m_Formatdd.FormatFlags = StringFormatFlags.NoWrap;
            m_Formatdd.LineAlignment = StringAlignment.Center;
            m_Formatdd.Trimming = StringTrimming.EllipsisCharacter;

            if (date.Date.Equals(DateTime.Now.Date))
            {
                DrawButton(g, rect, Color.FromArgb(255, 250, 237), Color.FromArgb(255, 236, 181), Color.FromArgb(229, 195, 101));
            }
            else
            {
                DrawGrayButton(g, rect);                
            }
            
            g.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;
            
            rect.Offset(0, 1);

            if (rect.Width < 2)
                return;


            Font fntDay = SystemFonts.IconTitleFont;
            
            string strDate;
            int StringWith = 0;

            try
            {
                strDate = date.ToString("D");
                StringWith = (int)g.MeasureString(strDate, fntDay).Width;

                if (StringWith > rect.Width - 5)
                {
                    strDate = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.GetDayName(date.DayOfWeek) + ", " + date.ToString("d");
                    StringWith = (int)g.MeasureString(strDate, fntDay).Width;

                    if (StringWith > rect.Width - 5)
                    {
                        strDate = System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.GetAbbreviatedDayName(date.DayOfWeek) + ", " + date.ToString("d");
                        StringWith = (int)g.MeasureString(strDate, fntDay).Width;

                        if (StringWith > rect.Width - 5)
                        {
                            strDate = date.ToString("d");
                            StringWith = (int)g.MeasureString(strDate, fntDay).Width;

                            if (StringWith > rect.Width - 5)
                            {
                                strDate = date.ToString("m");
                                StringWith = (int)g.MeasureString(strDate, fntDay).Width;

                                if (StringWith > rect.Width - 3)
                                {
                                    strDate = date.Day.ToString() + " " + System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.GetAbbreviatedMonthName(date.Month);
                                    StringWith = (int)g.MeasureString(strDate, fntDay).Width;

                                    if (StringWith > rect.Width - 3)
                                    {
                                        strDate = date.Day.ToString();
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception)
            {
                strDate = "?";                    
            }                
                
            g.DrawString(strDate, fntDay, SystemBrushes.WindowText, rect, m_Format);                        
        }

        public override void DrawDayBackground(Graphics g, Rectangle rect)
        {
            if (m_GanttControl != null)
                m_GanttControl.WeekendFormatStyle.BackgroundStyle.FillRectangle(g, rect);
        }

        public override void DrawAppointment(Graphics g, Rectangle rect, Appointment appointment, bool isSelected, int gripWidth, ImageList imageList)
        {           
            if (rect.Width == 0 || rect.Height == 0)
                //if (rect.Width < 6 || rect.Height < 3)
                return;


            StringFormat m_Format = new StringFormat();
            m_Format.Alignment = StringAlignment.Near;
            m_Format.LineAlignment = StringAlignment.Near;
            m_Format.Trimming = StringTrimming.EllipsisCharacter;

            //Color start = InterpolateColors(appointment.Color, Color.White, 0.4f);            
            //Color end = InterpolateColors(appointment.Color, Color.White, 0.7f);

            // better swap start and end color, text is more readable
            Color end = InterpolateColors(appointment.Color, Color.White, 0.4f);
            Color start = InterpolateColors(appointment.Color, Color.White, 0.7f);

            if ((appointment.Locked) || isSelected)
            {
                // Draw back                
                using (Brush m_Brush = new HatchBrush(HatchStyle.LargeConfetti, Color.FromArgb(70, 90, 125), appointment.Color))
                    g.FillRectangle(m_Brush, rect);

                // little transparent
                start = Color.FromArgb(230, start);
                end = Color.FromArgb(180, end);
                
                using (LinearGradientBrush aGB = new LinearGradientBrush(rect, start, end, LinearGradientMode.ForwardDiagonal))
                    g.FillRectangle(aGB, rect);
            }
            else
            {
                // Draw back
                using (LinearGradientBrush aGB = new LinearGradientBrush(rect, start, end, LinearGradientMode.Vertical))
                    g.FillRectangle(aGB, rect);
            }

            if (isSelected)
            {
                Rectangle m_BorderRectangle = rect;                
                m_BorderRectangle.Offset(-1, 0);

                using (Pen m_Pen = new Pen(appointment.BorderColor, 4))                
                    g.DrawRectangle(m_Pen, m_BorderRectangle);                
            }
            else
            {                            
                // Draw shadow lines
                int xLeft = rect.X + 6;
                int xRight = rect.Right + 1;
                int yTop = rect.Y + 1;
                int yButton = rect.Bottom + 1;

                for (int i = 0; i < 5; i++)
                {
                    using (Pen shadow_Pen = new Pen(Color.FromArgb(70 - 12 * i, Color.Black)))
                    {
                        g.DrawLine(shadow_Pen, xLeft + i, yButton + i, xRight + i - 1, yButton + i); //horisontal lines
                        g.DrawLine(shadow_Pen, xRight + i, yTop + i, xRight + i, yButton + i); //vertical
                    }
                }

                rect.Width -= 1;
                using (Pen m_Pen = new Pen(Color.FromArgb(70, 90, 125), 1))
                    g.DrawRectangle(m_Pen, rect);
                rect.Width += 1;
            }

            // Draw gripper *****
            Rectangle m_GripRectangle = rect;
            m_GripRectangle.Width = gripWidth + 1;

            using (SolidBrush aGB = new SolidBrush(appointment.BorderColor))
                g.FillRectangle(aGB, m_GripRectangle);
            
            
            // Draw Icons *****            
            rect.X += gripWidth + 2;
            rect.Width -= gripWidth + 2;

            if (rect.Height > 56)
            {
                rect.Y += 6;
                rect.Height -= 10;
            }
            else if (rect.Height > 38)
            {
                rect.Y += 4;
                rect.Height -= 8;
            }
            else
            {
                rect.Y += 3;
                rect.Height -= 6;
            }

            int LeftX = rect.X;
            int IconWidth = 0;
            if (appointment.ImageIndices != null && appointment.ImageIndices.Length > 0 && imageList != null && rect.Width >= 15 && rect.Height >= 10)
            {
                int imagecount = 0;

                int startX = rect.X;
                foreach (int ImageIndex in appointment.ImageIndices)
                {
                    imageList.Draw(g, new Point(startX, rect.Y), ImageIndex);
                    imagecount++;
                    IconWidth += 18;
                    startX = rect.X + (imagecount * 18);

                    if (startX + 18 > rect.Right - (gripWidth))
                    {
                        startX = rect.X;
                        rect.Y += 18;
                        rect.Height -= 18;
                        IconWidth = 0;

                        if (rect.Y + 18 > rect.Height)
                            break;
                    }
                }

                if (IconWidth > 0)
                    IconWidth += 2;
            }

            rect.Width -= 2;
            //rect.Y -= 1;

            // Draw Text *****
            SizeF TextSize = g.MeasureString(appointment.Title, this.BaseFont, rect.Width, m_Format);
            int TextLength = (int)TextSize.Width;
            int TextHeight = (int)TextSize.Height;

            if (TextLength + IconWidth + gripWidth + 2 > rect.Width && IconWidth > 20 && rect.Height > 36)
            {
                rect.Y += 20;
                rect.Height -= 20;
            }
            else
            {
                rect.X += IconWidth;
                rect.Width -= IconWidth;
            }

            g.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;
            
            g.DrawString(appointment.Title, this.BaseFont, SystemBrushes.WindowText, rect, m_Format);

            if (!String.IsNullOrEmpty(appointment.Description))
            {
                rect.X = LeftX;
                rect.Y += TextHeight + 16;
                rect.Height -= TextHeight + 14;
                
                if (rect.Height > 12)
                    g.DrawString(appointment.Description, this.BaseFont, SystemBrushes.WindowText, rect, m_Format);
            }            

            g.TextRenderingHint = TextRenderingHint.SystemDefault;

        }
    }
}
