﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;
using KS.Common;
using System.Drawing;

namespace ProjectPlanner
{
    static class Program
    {
        public static frmMain fMainForm;
        public static frmSplash fSplash;

        public static string DialogTitle = "KS-ProjectPlanner 2014";
        
        public static string VersionSuffix = "";

        public static string C_RegistrySectionKey = "Software\\Kroll-Software\\ProjectPlanner\\Settings";

        public static clsLanguageSelection LanguageSelection = null;
        public static System.Globalization.CultureInfo StartUpUICulture;

        public static SqlServer.GanttSqlDB SqlDB = null;

        public static object GetMySetting(string ValueKey, object DefaultValue)
        {
            object returnValue;
            KS.Common.KSRegistry cReg = new KS.Common.KSRegistry();
            cReg.sectionKey = C_RegistrySectionKey;
            cReg.valueKey = ValueKey;
            cReg.defaultValue = DefaultValue;
            returnValue = cReg.value;
            return returnValue;
        }

        public static void SaveMySetting(string ValueKey, object Value)
        {
            KS.Common.KSRegistry cReg = new KS.Common.KSRegistry();
            cReg.sectionKey = C_RegistrySectionKey;
            cReg.valueKey = ValueKey;
            cReg.value = Value;
        }        

        public static void CursorWait()
        {
            Cursor.Current = Cursors.WaitCursor;
        }

        public static void CursorDefault()
        {
            Cursor.Current = Cursors.Default;
        }

        public static void ErrMsgBox(string S)
        {
            MessageBox.Show(S, DialogTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        public static void InfoMsgBox(string S)
        {
            MessageBox.Show(S, DialogTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        public static DialogResult QuestionMsgBox(string S)
        {
            return MessageBox.Show(S, DialogTitle, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
        }

        public static void OpenURL(string strURL)
        {
            if (strURL != "")
            {
                try
                {
                    Process.Start(strURL);
                }
                catch (Exception ex)
                {
                    ErrMsgBox(ex.Message);
                }
            }
        }

        /// <summary>
        /// Der Haupteinstiegspunkt für die Anwendung.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            CursorWait();

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            StartUpUICulture = System.Threading.Thread.CurrentThread.CurrentUICulture;
            string strStartUpLanguageCode = StartUpUICulture.TwoLetterISOLanguageName;

            // workaround for sub-languages
            if (strStartUpLanguageCode == "zh")
                strStartUpLanguageCode = "zh-CHS";

            string strGUILanguage = GetMySetting("GUILanguage", strStartUpLanguageCode).SafeString();
            LanguageSelection = new clsLanguageSelection();

            try
            {
                if (LanguageSelection.LanguageCodes.Contains(strGUILanguage))
                    System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo(strGUILanguage);
                else
                    System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo("en");
            }
            catch (Exception)
            {
                System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo("en");
            }

            if (Program.GetMySetting("GUIFormats", 0).SafeBool())
            {
                System.Threading.Thread.CurrentThread.CurrentCulture = System.Threading.Thread.CurrentThread.CurrentUICulture;
            }            

            if (!(args.Length > 0 && String.Join(" ", args).IndexOf("nosplash") >= 0))
            {
                fSplash = new frmSplash();
                fSplash.Show();
                fSplash.Update();

                // Display our beautiful splash screen for 1,5 seconds
                for (int i = 1; i < 75; i++)
                {
                    System.Threading.Thread.Sleep(20);
                    Application.DoEvents();
                }
            }            

            fMainForm = new frmMain();

            if (fSplash != null)
            {
                fSplash.Close();
                fSplash.Dispose();
            }

            Application.Run(fMainForm);
        }        
    }
}