/*
{*******************************************************************}
{                                                                   }
{       KS-GANTT DotNet Library                                     }
{       Gantt-Charting Control for Windows Forms                    }
{                                                                   }
{       Copyright (c) 2009 - 2014 by Kroll-Software                 }
{       All Rights Reserved                                         }
{                                                                   }
{   This code is provided 'as is',                                  }
{   without warranties of any kind. Use at your own risk.           }                                                              
{                                                                   }
{   As a license holder of the KS-Gantt Control,                    }
{   you may freely redistribute executables based on                }
{   and created with these contents.                                }
{   You can modify the source code to meet your needs.              }
{                                                                   }
{   However, you must not publish, redistribute or transfer         }
{   this source code to 3rd parties under any conditions.           }
{                                                                   } 
{*******************************************************************}
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using KS.Common;
using KS.Gantt;
using ProjectPlanner.Clipboard;

namespace ProjectPlanner
{
    public partial class frmMain : Form, IMRUClient
    {
        private MRUManager mruManager;
        private ClipboardChangeNotifier clipChange = null;

        private ContextMenuStrip m_ContextMenu = null;

        public frmMain()
        {
            // Set Studio Colors for Gantt-Grids (blue hatched background, alternating row colors, etc)
            // also see single color options in KS.Gantt.Dialogs.Globals
            // This is static global, so we must set it before a Gantt Control is created.
            KS.Gantt.Dialogs.Globals.SetStudioColors();

            // when you obtained a license and only when KS.Gantt is used inside a Library (!), 
            // call this global static before any constructor is called
            // KS.Gantt.Helpers.LicenseResourceAssembly = System.Reflection.Assembly.GetAssembly(typeof(frmMain));

            InitializeComponent();
            
            if (!this.DesignMode)
                this.Font = SystemFonts.IconTitleFont;

            // Test
            //ucGantt1.gantt1.SelectionMode = Gantt.SelectionModes.SingleSelection;            

            ucGantt1.gantt1.SuspendItemLayout();            

            ucGantt1.ganttDataGrid1.SelectionMode = GanttDataGrid.GanttDataGridSelectionModes.Cell;            

            ucGantt1.gantt1.ItemsChanged += new EventHandler(gantt1_ItemsChanged);
            ucGantt1.gantt1.SelectionChanged += new Gantt.SelectionChangedEventHandler(gantt1_SelectionChanged);

            ucGantt1.ganttDataGrid1.UpdateEditMenu += new EventHandler(ganttDataGrid1_UpdateEditMenu);
            ucGantt1.gantt1.ViewModeChanged += new EventHandler(gantt1_ViewModeChanged);
            ucGantt1.gantt1.ItemDoubleClick += new KS.Gantt.Gantt.ItemDoubleClickEventHandler(gantt1_ItemDoubleClick);
            ucGantt1.gantt1.BeforeDelete += new KS.Gantt.Gantt.BeforeDeleteEventHandler(gantt1_BeforeDelete);

            ucGantt1.gantt1.MovingItem += new KS.Gantt.Gantt.MovingItemEventHandler(gantt1_MovingItem);
            ucGantt1.gantt1.ItemsMoved += new EventHandler(gantt1_ItemsMoved);
            ucGantt1.gantt1.ItemsResized += new EventHandler(gantt1_ItemsResized);
            ucGantt1.gantt1.ItemHover += new KS.Gantt.Gantt.ItemHoverEventHandler(gantt1_ItemHover);            

            ucGantt1.gantt1.GridLines = false;
            ucGantt1.gantt1.ShowWeekends = false;
            ucGantt1.gantt1.ShowWorkingHours = true;

            // SetUp SettingsDelegates            
            KS.Gantt.Dialogs.Globals.GetSetting = delegate(string key)
            {
                return Program.GetMySetting(key, "").SafeString();
            };

            KS.Gantt.Dialogs.Globals.SaveSetting = delegate(string key, string value)
            {
                Program.SaveMySetting(key, value);
            };

            SetGanttDefaults();

            mnuSelectAll.Click += new EventHandler(mnuSelectAll_Click);
            mnuInverseSelection.Click += new EventHandler(mnuInverseSelection_Click);

            mnuCollapseGroups.Click += new EventHandler(mnuCollapseGroups_Click);
            mnuExpandGroups.Click += new EventHandler(mnuExpandGroups_Click);

            mnuUndo.Click += new EventHandler(mnuUndo_Click);
            mnuUndo1.Click += new EventHandler(mnuUndo_Click);
            mnuRedo.Click += new EventHandler(mnuRedo_Click);
            mnuRedo1.Click += new EventHandler(mnuRedo_Click);

            mnuCut.Click += new EventHandler(mnuCut_Click);
            mnuCopy.Click += new EventHandler(mnuCopy_Click);
            mnuPaste.Click += new EventHandler(mnuPaste_Click);
            mnuDelete.Click += new EventHandler(mnuDelete_Click);
                        
            mnuTaskView.Click += new EventHandler(mnuTaskView_Click);            
            mnuResourceView.Click += new EventHandler(mnuResourceView_Click);
            
            mnuExit.Click += new EventHandler(mnuExit_Click);
            mnuZoomToFit.Click += new EventHandler(mnuZoomToFit_Click);
            mnuCenterNow.Click += new EventHandler(mnuCenterNow_Click);
            mnuZoomIn.Click += new EventHandler(mnuZoomIn_Click);
            mnuZoomOut.Click += new EventHandler(mnuZoomOut_Click);

            mnuAddTask.Click += new EventHandler(mnuAddTask_Click);
            mnuAddTask1.Click += new EventHandler(mnuAddTask_Click);
            mnuAddGroup.Click += new EventHandler(mnuAddGroup_Click);
            mnuAddGroup1.Click += new EventHandler(mnuAddGroup_Click);
            mnuAddResource.Click += new EventHandler(mnuAddResource_Click);
            mnuAddResource1.Click += new EventHandler(mnuAddResource_Click);

            mnuNew1.Click += new EventHandler(mnuNew_Click);
            mnuNew.Click += new EventHandler(mnuNew_Click);
            mnuNewInstance.Click += new EventHandler(mnuNewInstance_Click);

            mnuOpen1.Click += new EventHandler(mnuOpen_Click);
            mnuOpen.Click += new EventHandler(mnuOpen_Click);
            mnuSave1.Click += new EventHandler(mnuSave_Click);
            mnuSave.Click += new EventHandler(mnuSave_Click);            
            mnuSaveAs.Click += new EventHandler(mnuSaveAs_Click);

            mnuExportAsImage.Click += new EventHandler(mnuExportAsImage_Click);
            mnuExportAsXML.Click += new EventHandler(mnuExportAsXML_Click);

            mnuImportFromXML.Click += new EventHandler(mnuImportFromXML_Click);

            mnuPrintPageSetup.Click += new EventHandler(mnuPrintPageSetup_Click);
            mnuPrintPreview.Click += new EventHandler(mnuPrintPreview_Click);
            mnuPrint.Click += new EventHandler(mnuPrint_Click);

            mnuAbout.Click += new EventHandler(mnuAbout_Click);
            mnuAbout1.Click += new EventHandler(mnuAbout_Click);

            mnuHelpIndex.Click += new EventHandler(mnuHelpIndex_Click);
            mnuHelpIndex1.Click += new EventHandler(mnuHelpIndex_Click);
            mnuKrollSoftwareWebsite.Click += new EventHandler(mnuKrollSoftwareWebsite_Click);

            mnuProjectSettings.Click += new EventHandler(mnuProjectSettings_Click);
            mnuProjectSettings1.Click += new EventHandler(mnuProjectSettings_Click);

            mnuProjectCalendars.Click += new EventHandler(mnuProjectCalendars_Click);
            mnuProjectCalendars1.Click += new EventHandler(mnuProjectCalendars_Click);

            mnuProjectCost.Click += new EventHandler(mnuProjectCost_Click);
            mnuProjectCost1.Click += new EventHandler(mnuProjectCost_Click);

            mnuASAP.Click += new EventHandler(mnuASAP_Click);
            mnuASAP1.Click += new EventHandler(mnuASAP_Click);

            mnuSolve.Click += new EventHandler(mnuSolve_Click);
            mnuSolve1.Click += new EventHandler(mnuSolve_Click);            

            mnuPathDimming.Click += new EventHandler(mnuPathDimming_Click);
            ucGantt1.gantt1.PathDimmingChanged += new EventHandler(gantt1_PathDimmingChanged);                        

            mnuMaxZoomLevelGranularity.Checked = true;
            mnuMaxZoomLevelGranularity.Click += new EventHandler(mnuMaxZoomLevelGranularity_Click);
            mnuMaxZoomLevelMinute.Click += new EventHandler(mnuMaxZoomLevelMinute_Click);
            ucGantt1.gantt1.MaxZoomLevelChanged += new EventHandler(gantt1_MaxZoomLevelChanged);

            mnuSqlServerConnect.Click += new EventHandler(mnuSqlServerConnect_Click);
            mnuSqlServerConnect1.Click += new EventHandler(mnuSqlServerConnect_Click);
            mnuSqlServerDisconnect.Click += new EventHandler(mnuSqlServerDisconnect_Click);
            mnuSqlServerDisconnect1.Click += new EventHandler(mnuSqlServerDisconnect_Click);
            mnuSqlServerDeleteProject.Click += new EventHandler(mnuSqlServerDeleteProject_Click);

            mnuSQLCreateNewDatabase.Click += new EventHandler(mnuSQLCreateNewDatabase_Click);
            mnuSQLDropDatabase.Click += new EventHandler(mnuSQLDropDatabase_Click);

            mnuFind.Click += new EventHandler(mnuFind_Click);
            mnuFindMore.Click += new EventHandler(mnuFindMore_Click);
            mnuFind1.Click += new EventHandler(mnuFind_Click);
            mnuFindMore1.Click += new EventHandler(mnuFindMore_Click);

            mnuSqlServerDisconnect1.Visible = false;

            mnuSQLConnectOnStartUp.Click += new EventHandler(mnuSQLConnectOnStartUp_Click);
            mnuSQLConnectOnStartUp.Checked = Program.GetMySetting("SQLConnectOnStartUp", 0).SafeInt() == 1;            

            ucGantt1.gantt1.ResumeItemLayout();

            ucGantt1.gantt1.HistoryChanged += new EventHandler(gantt1_HistoryChanged);            
            ucGantt1.gantt1.ShowStatus += new Gantt.ShowStatusEventHandler(gantt1_ShowStatus);

            tabMain.SelectedIndexChanged += new EventHandler(tabMain_SelectedIndexChanged);

            SetUpContextMenu();

            // Test Topo
            ucGantt1.gantt1.CustomItemTextDisplay += new Gantt.ItemTextDisplayEventHandler(gantt1_CustomItemTextDisplay);            

            this.KeyDown += new KeyEventHandler(frmMain_KeyDown);            

            // *************** Load ******************            

            ucTasks1.GanttControl = ucGantt1.gantt1;
            ucResources1.GanttControl = ucGantt1.gantt1;
            ucGroups1.GanttControl = ucGantt1.gantt1;
            ucSkills1.GanttControl = ucGantt1.gantt1;
            ucSchedule1.GanttControl = ucGantt1.gantt1;

            ucGantt1.gantt1.UndoRedoCalled += new EventHandler(gantt1_UndoRedoCalled);

            InitLanguageMenu();            

            // Set some language specific texts
            ucGantt1.gantt1.NewTaskName = Properties.Resources.NewTaskName;
            ucGantt1.gantt1.WeekPrefix = Properties.Resources.WeekPrefix;

            ucGantt1.gantt1.PathDimming = Program.GetMySetting("PathDimming", 1).SafeBool();

            bool bMaxZoomLevelMinute = Program.GetMySetting("MaxZoomLevelMinute", 1).SafeBool();
            if (bMaxZoomLevelMinute)
                ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Minute;
            else
                ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Granularity;

            KS.Gantt.Helpers.DialogTitle = Program.DialogTitle;

            // MRU FileList
            mruManager = new MRUManager();
            mruManager.Initialize(
             this,                              // owner form
             mnuFileMRU,                        // Recent Files menu item
             Program.C_RegistrySectionKey);     // Registry path to keep MRU list

            // Optional. Call these functions to change default values:
            //mruManager.CurrentDir = ".....";           // default is current directory
            mruManager.MaxMRULength = 7;             // default is 10
            mruManager.MaxDisplayNameLength = 60;     // default is 40

            // Some Shortcuts
            // Setting Shortcut Keys by resource, throws errors in different Visual-Studio language versions.
            // This is, why we set them by code
            
            mnuNew.ShortcutKeys = Keys.N | Keys.Control;
            mnuNewInstance.ShortcutKeys = Keys.N | Keys.Control | Keys.Alt;
            mnuOpen.ShortcutKeys = Keys.O | Keys.Control;
            mnuSave.ShortcutKeys = Keys.S | Keys.Control;
            mnuSaveAs.ShortcutKeys = Keys.S | Keys.Control | Keys.Alt;
            mnuPrint.ShortcutKeys = Keys.P | Keys.Control;

            mnuTaskView.ShortcutKeys = Keys.T | Keys.Control;
            mnuResourceView.ShortcutKeys = Keys.R | Keys.Control;
            
            mnuZoomToFit.ShortcutKeys = Keys.M | Keys.Control;
            mnuCenterNow.ShortcutKeys = Keys.M | Keys.Alt;

            mnuSelectAll.ShortcutKeys = Keys.A | Keys.Control;
            
            mnuUndo.ShortcutKeys = Keys.Z | Keys.Control;
            mnuRedo.ShortcutKeys = Keys.Y | Keys.Control;            

            mnuDelete.ShortcutKeys = Keys.Delete;
            mnuCut.ShortcutKeys = Keys.X | Keys.Control;
            mnuCopy.ShortcutKeys = Keys.C | Keys.Control;
            mnuPaste.ShortcutKeys = Keys.V | Keys.Control;

            mnuFind.ShortcutKeys = Keys.F | Keys.Control;
            mnuASAP.ShortcutKeys = Keys.L | Keys.Control;

            mnuExpandGroups.ShortcutKeys = Keys.Oemplus | Keys.Control;
            mnuCollapseGroups.ShortcutKeys = Keys.OemMinus | Keys.Control;

            // Setup GanttControl for the Schedule-Renderer
            // By this, the Schedule-Renderer picks the Weekend Colors from the GanttControl
            this.ucSchedule1.dayView1.Renderer.GanttControl = ucGantt1.gantt1;

            // Setup some colors for the GanttDataGrid
            if (true)   // set false to see the Gantt-Control's default colors
            {
                ucGantt1.ganttDataGrid1.HeaderBackground.Color = ucSchedule1.dayView1.Renderer.HoursBackColor;
                ucGantt1.ganttDataGrid1.HeaderBackground.SetDefault();

                ucGantt1.ganttDataGrid1.HeaderForeground.Color = ucGantt1.gantt1.HeaderManager.MonthHeader.FormatStyle.ForegroundStyle.Color;
                ucGantt1.ganttDataGrid1.HeaderForeground.SetDefault();

                ucGantt1.gantt1.HeaderBackgroundStyle.Color = ucSchedule1.dayView1.Renderer.HoursBackColor;
                ucGantt1.gantt1.HeaderBackgroundStyle.SetDefault();

                ucGantt1.gantt1.HeaderManager.YearHeader.FormatStyle.BackgroundStyle.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.gantt1.HeaderManager.YearHeader.FormatStyle.SetDefault();

                ucGantt1.gantt1.HeaderManager.MonthHeader.FormatStyle.BackgroundStyle.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.gantt1.HeaderManager.MonthHeader.FormatStyle.SetDefault();

                ucGantt1.gantt1.HeaderManager.WeekHeader.FormatStyle.BackgroundStyle.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.gantt1.HeaderManager.WeekHeader.FormatStyle.SetDefault();
            }

            // do this at last
            ucGantt1.gantt1.ResetModified();
            ucGantt1.gantt1.HistoryClear();


#if (DEBUG)
            System.Windows.Forms.ToolStripMenuItem mnuDoSelfTests = new ToolStripMenuItem("Do Self-Tests");
            mnuDoSelfTests.Click += mnuDoSelfTests_Click;
            mnuExtras.DropDownItems.Add(mnuDoSelfTests);
#endif
        }

        void InitLanguageMenu()
        {
            if (!this.DesignMode)
            {
                foreach (string strLanguageCode in Program.LanguageSelection.LanguageCodes)
                {
                    ToolStripMenuItem cmdLanguage = new ToolStripMenuItem(Program.LanguageSelection.LanguageNames[strLanguageCode]);
                    cmdLanguage.Name = "mnuLanguage" + strLanguageCode;
                    cmdLanguage.Tag = strLanguageCode;

                    mnuLanguage.DropDownItems.Add(cmdLanguage);
                    cmdLanguage.Click += new EventHandler(cmdLanguage_Click);

                    if (System.Threading.Thread.CurrentThread.CurrentUICulture.Name.StartsWith(Strings.StrLeft(strLanguageCode, 2)))
                        cmdLanguage.Checked = true;
                }

                mnuLanguage.DropDownItems.Add(new ToolStripSeparator());

                ToolStripMenuItem mnuCulture = new ToolStripMenuItem(Properties.Resources.GuiNumberformats);
                mnuCulture.Click += new EventHandler(mnuCulture_Click);
                mnuCulture.Checked = Program.GetMySetting("GUIFormats", 0).SafeBool();
                mnuLanguage.DropDownItems.Add(mnuCulture);
            }
        }

        void mnuCulture_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem cmd = (ToolStripMenuItem)sender;
            cmd.Checked = !cmd.Checked;
            Program.SaveMySetting("GUIFormats", cmd.Checked.ToInt());
            Program.InfoMsgBox(Properties.Resources.RestartApplication);
        }

        void cmdLanguage_Click(object sender, EventArgs e)
        {
            ChangeLanguage((sender as ToolStripMenuItem).Tag.SafeString());
        }

        private void ChangeLanguage(string GUILanguage)
        {
            Program.SaveMySetting("GUILanguage", GUILanguage);
            Program.InfoMsgBox(Properties.Resources.RestartApplication);

            foreach (ToolStripItem cmd in mnuLanguage.DropDownItems)
            {
                if (cmd is ToolStripMenuItem && cmd.Tag != null)
                {
                    ((ToolStripMenuItem)cmd).Checked = cmd.Tag.SafeString() == GUILanguage;
                }
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.Text = Program.DialogTitle;

            if (Program.GetMySetting("MainMaximized", 0).SafeBool())
            {
                this.WindowState = FormWindowState.Maximized;
            }
            else
            {
                this.Left = Program.GetMySetting("MainLeft", 50).SafeInt(50);
                this.Top = Program.GetMySetting("MainTop", 50).SafeInt(50);
                this.Width = Program.GetMySetting("MainWidth", 800).SafeInt(800);
                this.Height = Program.GetMySetting("MainHeight", 600).SafeInt(600);
            }

            if (this.Width < 500)
                this.Width = 500;

            if (this.Height < 400)
                this.Height = 400;

            ucGantt1.splitContainer1.SplitterDistance = Program.GetMySetting("GanttSplitterWidth", 190).SafeInt(190);

            // SetUp the Clipboard Monitor
            this.clipChange = new ClipboardChangeNotifier();
            this.clipChange.ClipboardChanged += new EventHandler(clipChange_ClipboardChanged);
            this.clipChange.AssignHandle(this.Handle);
            this.clipChange.Install();

            UpdateMenu();
            ShowStatus();

            // Connect DB on StartUp
            if (Program.GetMySetting("SQLConnectOnStartUp", 0).SafeInt() == 1)
            {                
                mnuSqlServerConnect.PerformClick();
            }
        }

        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);

            // Open File from the Command Line
            string[] args = Environment.GetCommandLineArgs();
            string FName = "";
            if (args.Length >= 2)
                FName = args[1];

            if (FName != "" && KS.Common.Strings.FileExists(FName))
            {
                Application.DoEvents();
                OpenFile(FName);
            }
        }

        void gantt1_ShowStatus(object sender, GanttStatusEventArgs e)
        {
            if (e.Level == 0)
            {
                ShowStatus(e.StatusText, e.ShowWaitCursor);
                
                if (bEscPressed)
                    e.Cancel = true;
            }
            else
            {
                toolStripStatusLabelTaskName.Text = e.StatusText;
                statusStrip1.Update();
            }            
        }

        private void frmMain_Load(object sender, EventArgs e)
        {            
        }        

        // Open SQL-Server on StartUp
        void mnuSQLConnectOnStartUp_Click(object sender, EventArgs e)
        {
            mnuSQLConnectOnStartUp.Checked = !mnuSQLConnectOnStartUp.Checked;

            if (mnuSQLConnectOnStartUp.Checked)
                Program.SaveMySetting("SQLConnectOnStartUp", 1);
            else
                Program.SaveMySetting("SQLConnectOnStartUp", 0);

        }        

        // *** Path Dimming ***
        void mnuPathDimming_Click(object sender, EventArgs e)
        {            
            ucGantt1.gantt1.PathDimming = !mnuPathDimming.Checked;
        }

        void gantt1_PathDimmingChanged(object sender, EventArgs e)
        {            
            UpdateMenu();
            if (ucGantt1.gantt1.PathDimming)
                Program.SaveMySetting("PathDimming", 1);
            else
                Program.SaveMySetting("PathDimming", 0);
        }        

        // *** Max Zoom Level ***
        void mnuMaxZoomLevelGranularity_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Granularity;
        }

        void mnuMaxZoomLevelMinute_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Minute;
        }

        void gantt1_MaxZoomLevelChanged(object sender, EventArgs e)
        {
            UpdateMenu();
            if (ucGantt1.gantt1.MaxZoomLevel == Gantt.ZoomLevels.Granularity)
                Program.SaveMySetting("MaxZoomLevelMinute", 0);
            else
                Program.SaveMySetting("MaxZoomLevelMinute", 1);
        }        

        private bool GanttVisible
        {
            get
            {
                return tabMain.SelectedIndex < 2;
            }
        }

        private bool bEscPressed = false;
        void frmMain_KeyDown(object sender, KeyEventArgs e)
        {
            if (ucGantt1.ganttDataGrid1.IsEditing)
                return;                        

            switch (e.KeyCode)
            {
                case Keys.Delete:
                    if (GanttVisible)
                    {                        
                        if (e.Shift || e.Control)
                            ucGantt1.ganttDataGrid1.Cut();
                        else
                            ucGantt1.ganttDataGrid1.Delete();

                        e.Handled = true;
                    }
                    break;
                
                case Keys.Insert:
                    if (GanttVisible)
                    {                        
                        if (e.Control)
                        {
                            ucGantt1.ganttDataGrid1.Copy();
                            e.Handled = true;
                        }
                        else if (e.Shift)
                        {
                            ucGantt1.ganttDataGrid1.Paste();
                            e.Handled = true;
                        }                        
                    }
                    break;                

                case Keys.Escape:                    
                    bEscPressed = true;
                    break;
            }
        }        

        void mnuNewInstance_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process instance = null;

            try
            {
                Program.CursorWait();
                instance = new System.Diagnostics.Process();
                instance.StartInfo.FileName = Application.ExecutablePath;
                instance.StartInfo.Arguments = "nosplash";
                instance.Start();
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {
                if (instance != null)
                    instance.Dispose();

                Program.CursorDefault();
            }
        }        
        
        void gantt1_CustomItemTextDisplay(object sender, GanttItemTextEventArgs e)
        {
            bool ShowKeys = false;

            if (e.Item.ItemType == GanttItemType.TaskItem)
            {                
                e.Text = e.Item.Text;

                if (ShowKeys)
                    e.Text += " (" + e.Item.Key + ")";

                e.Text += " - Topo: " + ((TaskItem)e.Item).topo_index.ToString();
                e.Text += " - Path: " + ((TaskItem)e.Item).connectedcomponent_index.ToString();
                //e.Text += " - Distance: " + ((TaskItem)e.Item).cpm_Distance.ToString("n2");                

                if (ShowKeys)
                {                    
                    if (e.Item.ParentItemKey != null)
                        e.Text += " - ParentItemKey: " + e.Item.ParentItemKey;
                    else if (e.Item.ParentItem != null)
                        e.Text += " - ParentItem.Key: " + e.Item.ParentItem.Key;
                }

            }
            else if (e.Item.ItemType == GanttItemType.GroupItem)
            {
                e.Text = e.Item.Text;

                if (ShowKeys)
                    e.Text += " (" + e.Item.Key + ")";
            }
        }        

        void tabMain_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabMain.SelectedIndex < 2)
                ucGantt1.Parent = tabMain.SelectedTab;

            switch (tabMain.SelectedTab.Name)
            {
                case "tabGantt":
                    ucGantt1.gantt1.ViewMode = Gantt.ViewModes.Tasks;
                    break;

                case "tabResourceView":
                    ucGantt1.gantt1.ViewMode = Gantt.ViewModes.Resources;
                    break;

                default:
                    break;
            }

            UpdateMenu();
        }        

        void gantt1_HistoryChanged(object sender, EventArgs e)
        {
            if (!ucGantt1.gantt1.IsItemLayoutSuspended)
                UpdateMenu();
        }                

        void mnuProjectCost_Click(object sender, EventArgs e)
        {
            // We could simply call gantt.CalcProjectCost()
            // but we want to consider selected items, if there are any
            // and also consider the selected item in ResourceView            

            double cost = 0.0;

            bool bSelected = false;

            KS.Gantt.Gantt gantt = ucGantt1.gantt1;
            if (gantt.ViewMode == Gantt.ViewModes.Tasks)
            {
                // Filter selected tasks, if there are any, otherwise calc on all tasks
                bSelected = gantt.SelectedItems.Count > 0;
                cost = gantt.CalcProjectCost(bSelected);
            }
            else
            {
                // SelectedItems is only valid for the TaskView
                // in ResourceView there might be a single 'LastSelectedItem'
                //
                // In ResourceView, there is an extra TaskItemCollection called 'ResourceBars'
                // They are constructed of 'ResourceHeaders' and of simple TaskItems, that are clones of the original tasks
                // A ResourceHearder's Parent is the depending ResourceItem

                GanttItem SelectedItem = gantt.LastSelectedItem;
                if (SelectedItem == null)
                {
                    // calc cost of the entire project 
                    cost = gantt.CalcProjectCost();
                }                
                else if (SelectedItem is TaskItem)
                {
                    bSelected = true;

                    TaskItem task = (TaskItem)SelectedItem;
                    if (task.IsResourceHeaderItem && task.ParentItem != null && task.ParentItem.ItemType == GanttItemType.ResourceItem)
                        cost = gantt.CalcProjectCost((ResourceItem)task.ParentItem);
                    else
                        cost = gantt.CalcProjectCost("", task.Key, false);
                }                
            }            

            string strMessage = Properties.Resources.ProjectCost + ":\r\n";
            if (bSelected)
                strMessage += Properties.Resources.SelectedElements + "\r\n";
            strMessage += "\r\n" + cost.ToString("c");
            
            Program.InfoMsgBox(strMessage);
        }

        void mnuASAP_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.AlignProjectASAP();
        }

        void mnuSolve_Click(object sender, EventArgs e)
        {
            bEscPressed = false;

            if (SqlDBConncted)
            {
                try
                {
                    ShowStatus(Properties.Resources.SQLRefreshingResourceAllocations, true);
                    Program.SqlDB.RefreshResourceAllocationsFromOtherProjects(ucGantt1.gantt1);
                }
                catch (Exception ex)
                {                    
                    Program.ErrMsgBox(ex.Message);
                    return;
                }
                finally
                {
                    ShowStatus();
                }
            }

            ucGantt1.gantt1.ShowSolveDialog();            
        }        

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {            
            EnableGUI(true);
            bEscPressed = true;

            if (!AskSave())
            {                
                e.Cancel = true;
                return;                
            }

            if ((this.WindowState != FormWindowState.Minimized) && (this.WindowState != FormWindowState.Maximized))
            {
                Program.SaveMySetting("MainLeft", this.Left);
                Program.SaveMySetting("MainTop", this.Top);
                Program.SaveMySetting("MainWidth", this.Width);
                Program.SaveMySetting("MainHeight", this.Height);
            }

            Program.SaveMySetting("GanttSplitterWidth", ucGantt1.splitContainer1.SplitterDistance);

            if (ucSchedule1.Created)
                Program.SaveMySetting("ScheduleSplitterWidth", ucSchedule1.splitContainer1.Width - ucSchedule1.splitContainer1.SplitterDistance);

            if (this.WindowState == FormWindowState.Maximized)
                Program.SaveMySetting("MainMaximized", 1);
            else
                Program.SaveMySetting("MainMaximized", 0);            

            this.Hide();
        }

        void mnuAbout_Click(object sender, EventArgs e)
        {            
            frmAbout Dlg = new frmAbout();
            Dlg.ShowDialog(this);
            Dlg.Dispose();
        }

        void mnuHelpIndex_Click(object sender, EventArgs e)
        {
            modHelp.ShowHelp(modHelp.HelpKeys.CTX_HelpIndex);
        }

        void mnuKrollSoftwareWebsite_Click(object sender, EventArgs e)
        {
            Program.OpenURL(Properties.Resources.KrollWebsiteUrl);
        }

        bool EndEdit()
        {
            if (!ucTasks1.EndEdit())
            {
                tabMain.SelectedTab = tabTasks;
                return false;
            }

            if (!ucResources1.EndEdit())
            {
                tabMain.SelectedTab = tabResources;
                return false;
            }

            if (!ucGroups1.EndEdit())
            {
                tabMain.SelectedTab = tabGroups;
                return false;
            }

            if (!ucSkills1.EndEdit())
            {
                tabMain.SelectedTab = tabSkills;
                return false;
            }

            if (!ucGantt1.ganttDataGrid1.EndEdit(true))
            {
                tabMain.SelectedTab = tabGantt;
                return false;
            }

            return true;
        }

        private void ResetAllBindings()
        {            
            ucTasks1.InitData();
            ucResources1.InitData();
            ucGroups1.InitData();
            ucSkills1.InitData();
            ucSchedule1.InitData();
        }        

        void gantt1_UndoRedoCalled(object sender, EventArgs e)
        {            
            ResetAllBindings();            
        }

        private void SetGanttDefaults()
        {
            // Gantt-Defaults
            ucGantt1.gantt1.SetDefaultStyles();
            ucGantt1.gantt1.WeekendFormatStyle.BackgroundStyle.Gradient = false;

            ucGantt1.gantt1.DefaultDurationType = TaskItem.DurationTypes.FullWorkingDays;
            ucGantt1.gantt1.DefaultTaskType = TaskItem.TaskTypes.FixedUnits;
            ucGantt1.gantt1.Granularity = Granularities.QuarterHour;
            ucGantt1.gantt1.ResourceViewFilter = Gantt.ResourceViewFilters.AssignedResources;

            ucGantt1.gantt1.GridLines = false;
            ucGantt1.gantt1.ItemTextDisplay = Gantt.ItemTextDisplayModes.TextAndResources;

            ucGantt1.gantt1.BarHeight = 14;
            ucGantt1.gantt1.RowHeight = 24;

            ucGantt1.gantt1.ShowIcons = false;

            ucGantt1.gantt1.StartDate = DateTime.Now.AddMonths(-1);
            ucGantt1.gantt1.EndDate = DateTime.Now.AddMonths(1);

            ucGantt1.gantt1.ViewMode = Gantt.ViewModes.Tasks;
            ucGantt1.gantt1.ShowIcons = false;

            ucGantt1.ganttDataGrid1.ShowItemCalendarOnRowChange = true;
        }

        void mnuNew_Click(object sender, EventArgs e)
        {
            if (tabMain.SelectedIndex > 1)
                tabMain.SelectedTab = tabGantt;

            if (!AskSave())
                return;

            ucGantt1.gantt1.SuspendItemLayout();

            ucGantt1.gantt1.ClearData();            
            SetGanttDefaults();

            if (SqlDBConncted)
            {
                ShowStatus(Properties.Resources.Status_ReadingSQL, true);

                try
                {
                    Program.SqlDB.ReadDBGlobal(ucGantt1.gantt1);    // read global data as calendars, skills and resources
                }
                catch (Exception ex)
                {
                    Program.CursorDefault();
                    Program.ErrMsgBox(ex.Message);
                    return;
                }
                finally
                {
                    ShowStatus();
                }                               

                SQLDBOpenFlag = false;
            }            
                        
            ucGantt1.gantt1.HistoryClear();
            ucGantt1.gantt1.ResetModified();

            ucGantt1.gantt1.ResumeItemLayout();

            tabMain.SelectedTab = tabGantt;
            ResetAllBindings();
                        
            ProjectFilename = "";            
            UpdateMenu();
        }

        private string ProjectFilename = "";
        private bool SQLDBOpenFlag = false;

        void mnuOpen_Click(object sender, EventArgs e)
        {           
            if (!AskSave())
                return;

            if (SqlDBConncted)
            {
                Program.CursorWait();

                string ProjectKey = "";

                SqlServer.SelectProjectDialog Dlg = new ProjectPlanner.SqlServer.SelectProjectDialog();                
                Dlg.ProjectKey = ucGantt1.gantt1.Project.Key;
                Dlg.GanttDB = Program.SqlDB;
                if (Dlg.ShowDialog(this) == DialogResult.OK)                
                {
                    ProjectKey = Dlg.ProjectKey;
                }                

                Dlg.Dispose();

                if (ProjectKey != "")
                {
                    tabMain.SelectedTab = tabGantt;
                    this.Update();                    

                    try
                    {
                        ShowStatus(Properties.Resources.Status_ReadingSQL, true);
                        Program.SqlDB.ReadDB(ucGantt1.gantt1, ProjectKey);
                    }
                    catch (Exception ex)
                    {
                        Program.CursorDefault();
                        Program.ErrMsgBox(ex.Message);
                        return;
                    }
                    finally
                    {
                        ShowStatus();
                    }                                       

                    SQLDBOpenFlag = true;
                    ResetAllBindings();
                    UpdateMenu();
                }
            }
            else
            {
                string FName;

                System.Windows.Forms.OpenFileDialog FileDialog = new System.Windows.Forms.OpenFileDialog();
                FileDialog.Title = Properties.Resources.Caption_Open;
                FileDialog.Filter = Properties.Resources.FileDialog_PPP_Filter;
                FileDialog.CheckFileExists = true;
                FileDialog.Multiselect = false;
                FileDialog.DereferenceLinks = true;
                FileDialog.FilterIndex = 1;
                FileDialog.FileName = "";
                if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
                {
                    FileDialog.Dispose();
                    return;
                }

                FName = FileDialog.FileName;
                FileDialog.Dispose();

                OpenFile(FName);
            }

            ucGantt1.gantt1.ShowIcons = false;
        }

        private void OpenFile(string FName)
        {
            tabMain.SelectedTab = tabGantt;
            this.Update();

            try
            {
                ShowStatus(Properties.Resources.Status_ReadingFile, true);
                ucGantt1.gantt1.DeSerializeData(FName);
                ProjectFilename = FName;
                mruManager.Add(FName);
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
                mruManager.Remove(FName);
            }
            finally
            {
                ResetAllBindings();

                ucGantt1.gantt1.HistoryClear();
                ucGantt1.gantt1.ResetModified();

                ShowStatus();
                UpdateMenu();
                SQLDBOpenFlag = false;
            }
        }

        bool AskSave()
        {
            if (!EndEdit())
                return false;

            if (!ucGantt1.gantt1.Modified)
                return true;
            
            switch (Program.QuestionMsgBox(Properties.Resources.Message_SaveChanges))
            {
                case DialogResult.Cancel:
                    return false;

                case DialogResult.No:
                    return true;

                case DialogResult.Yes:
                    return SaveProject(false);                    
            }

            throw new Exception("Unknown DialogResult");
        }

        public bool SaveProject(bool bSaveAs)
        {
            string FName = "";

            if (SqlDBConncted && !bSaveAs)
            {                
                string dbUser;
                if (Program.SqlDB.DBUser == "" || Program.SqlDB.DBWinSecurity)
                    dbUser = DatabaseHelpers.WindowsUser();
                else
                    dbUser = Program.SqlDB.DBUser;

                // Check, if this Project is new
                try
                {
                    if (ucGantt1.gantt1.Project.Text.Trim() == "" || !Program.SqlDB.ProjectExists(ucGantt1.gantt1.Project.Key))
                    {
                        SqlServer.ProjectNameDialog DlgName = new ProjectPlanner.SqlServer.ProjectNameDialog(ucGantt1.gantt1.Project);
                        bool bOK = DlgName.ShowDialog(this) == DialogResult.OK;
                        DlgName.Dispose();
                        DlgName = null;

                        if (!bOK)
                            return false;

                        ucGantt1.gantt1.Project.SetProperty("created", DateTime.Now);
                        ucGantt1.gantt1.Project.SetProperty("created by", dbUser);
                    }
                    
                    ucGantt1.gantt1.Project.SetProperty("changed", DateTime.Now);
                    ucGantt1.gantt1.Project.SetProperty("changed by", dbUser);
                }
                catch (Exception ex)
                {
                    Program.ErrMsgBox(ex.Message);
                    return false;
                }                

                ShowStatus(Properties.Resources.Status_WritingSQL, true);

                try
                {                    
                    // Write changes to the database
                    Program.SqlDB.UpdateDB(ucGantt1.gantt1);
                }
                catch (Exception ex)
                {
                    Program.CursorDefault();
                    Program.ErrMsgBox(ex.Message);
                    return false;
                }
                finally
                {
                    ShowStatus();
                }

                SQLDBOpenFlag = true;                               

                UpdateMenu();
            }
            else
            {
                if (bSaveAs || ProjectFilename == "")
                {
                    System.Windows.Forms.SaveFileDialog FileDialog = new System.Windows.Forms.SaveFileDialog();

                    if (bSaveAs)
                        FileDialog.Title = Properties.Resources.Caption_SaveAs;
                    else
                        FileDialog.Title = Properties.Resources.Caption_Save;

                    FileDialog.Filter = Properties.Resources.FileDialog_PPP_Filter;
                    FileDialog.FilterIndex = 1;
                    FileDialog.DefaultExt = ".ppp";
                    FileDialog.FileName = KS.Common.Strings.GetFilename(ProjectFilename);
                    if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
                    {
                        FileDialog.Dispose();
                        return false;
                    }

                    FName = FileDialog.FileName;
                    FileDialog.Dispose();
                    this.Update();

                    try
                    {
                        ShowStatus(Properties.Resources.Status_WritingFile, true);
                        ucGantt1.gantt1.SerializeData(FName);
                        ucGantt1.gantt1.SetHistoryModified();
                        ucGantt1.gantt1.ResetModified();
                        ProjectFilename = FName;
                    }
                    catch (Exception ex)
                    {
                        Program.CursorDefault();
                        Program.ErrMsgBox(ex.Message);
                        return false;
                    }
                    finally
                    {
                        ShowStatus();
                        UpdateMenu();
                    }
                }
                else
                {
                    try
                    {
                        ShowStatus(Properties.Resources.Status_WritingFile, true);
                        ucGantt1.gantt1.SerializeData(ProjectFilename);
                        ucGantt1.gantt1.SetHistoryModified();
                        ucGantt1.gantt1.ResetModified();
                    }
                    catch (Exception ex)
                    {
                        Program.CursorDefault();
                        Program.ErrMsgBox(ex.Message);
                        return false;
                    }
                    finally
                    {
                        ShowStatus();
                        UpdateMenu();
                    }
                }

                if (FName != "")
                    mruManager.Add(FName);
            }

            return true;
        }        

        void mnuSave_Click(object sender, EventArgs e)
        {
            SaveProject(false);
        }

        void mnuSaveAs_Click(object sender, EventArgs e)
        {
            SaveProject(true);
        }

        void mnuExportAsImage_Click(object sender, EventArgs e)
        {
            string FName = "";

            System.Windows.Forms.SaveFileDialog FileDialog = new System.Windows.Forms.SaveFileDialog();

            FileDialog.Title = Properties.Resources.Caption_ExportAsImage;

            FileDialog.Filter = Properties.Resources.ExportAsImage_Filter;
            FileDialog.FilterIndex = 1;
            FileDialog.DefaultExt = ".png";
            FileDialog.FileName = "";
            if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
            {
                FileDialog.Dispose();
                return;
            }

            FName = FileDialog.FileName;
            FileDialog.Dispose();
            this.Update();

            Bitmap bmp = null;

            try
            {
                bmp = ucGantt1.ganttDataGrid1.GetBitmap();

                if (bmp != null)
                {
                    bmp.Save(FName, System.Drawing.Imaging.ImageFormat.Png);
                }
            }
            catch (Exception ex)
            {
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {
                if (bmp != null)
                    bmp.Dispose();
            }            
        }

        void mnuExportAsXML_Click(object sender, EventArgs e)
        {
            string FName = "";

            System.Windows.Forms.SaveFileDialog FileDialog = new System.Windows.Forms.SaveFileDialog();

            FileDialog.Title = Properties.Resources.Caption_ExportAsXML;

            FileDialog.Filter = Properties.Resources.ExportAsXML_Filter;
            FileDialog.FilterIndex = 1;
            FileDialog.DefaultExt = ".xml";
            FileDialog.FileName = "";
            if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
            {
                FileDialog.Dispose();
                return;
            }

            FName = FileDialog.FileName;
            FileDialog.Dispose();
            this.Update();

            try
            {
                ShowStatus(Properties.Resources.Status_WritingFile, true);
                ucGantt1.gantt1.SerializeDataXML(FName);
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {                
                ShowStatus();
                UpdateMenu();                
            }
        }

        void mnuImportFromXML_Click(object sender, EventArgs e)
        {
            if (!AskSave())
                return;

            string FName;

            System.Windows.Forms.OpenFileDialog FileDialog = new System.Windows.Forms.OpenFileDialog();
            FileDialog.Title = Properties.Resources.Caption_ImportProjektPlannerXML;
            FileDialog.Filter = Properties.Resources.FileDialog_PPXML_Filter;
            FileDialog.CheckFileExists = true;
            FileDialog.Multiselect = false;
            FileDialog.DereferenceLinks = true;
            FileDialog.FilterIndex = 1;
            FileDialog.FileName = "";
            if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
            {
                FileDialog.Dispose();
                return;
            }

            FName = FileDialog.FileName;
            FileDialog.Dispose();
            this.Update();            

            try
            {
                ShowStatus(Properties.Resources.Status_ReadingFile, true);
                ucGantt1.gantt1.DeSerializeDataXML(FName, SqlDBConncted);

                ucGantt1.gantt1.SetModified();
                ProjectFilename = "";
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {                
                ShowStatus();
                UpdateMenu();                
            }
        }        

        void mnuPrintPageSetup_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Printing.PrintPageSetup();
        }

        void mnuPrintPreview_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Printing.PrintPreview();            
        }

        void mnuPrint_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Printing.Print();
        }

        void mnuProjectSettings_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ShowProjectSettingsDialog();
            UpdateMenu();            
        }

        void mnuProjectCalendars_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ShowCalendarListDialog();
            UpdateMenu();            
        }

        void mnuAddResource_Click(object sender, EventArgs e)
        {
            Program.CursorWait();
            KS.Gantt.Dialogs.ResourceEditDialog Dlg = new KS.Gantt.Dialogs.ResourceEditDialog();
            Dlg.Edit(ucGantt1.gantt1, null);
            Dlg.ShowDialog(this);
            Dlg.Dispose();
            Dlg = null;            
        }

        void mnuAddGroup_Click(object sender, EventArgs e)
        {
            Program.CursorWait();
            KS.Gantt.Dialogs.GroupEditDialog Dlg = new KS.Gantt.Dialogs.GroupEditDialog();
            Dlg.Edit(ucGantt1.gantt1, null);
            Dlg.ShowDialog(this);

            if (Dlg.OkPressed)
            {
                ucGantt1.gantt1.ViewMode = Gantt.ViewModes.Tasks;
                tabMain.SelectedTab = tabGantt;
            }

            Dlg.Dispose();
            Dlg = null;            
        }

        void mnuAddTask_Click(object sender, EventArgs e)
        {
            Program.CursorWait();
            KS.Gantt.Dialogs.TaskEditDialog Dlg = new KS.Gantt.Dialogs.TaskEditDialog();
            Dlg.Edit(ucGantt1.gantt1, null);
            Dlg.ShowDialog(this);

            if (Dlg.OkPressed)
            {
                ucGantt1.gantt1.ViewMode = Gantt.ViewModes.Tasks;
                tabMain.SelectedTab = tabGantt;
            }

            Dlg.Dispose();
            Dlg = null;            
        }

        void mnuZoomIn_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ZoomIn();
        }

        void mnuZoomOut_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ZoomOut();            
        }        

        void mnuCenterNow_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.CenterNow();            
        }

        void mnuZoomToFit_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ZoomToFit();            
        }

        void mnuExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        void mnuTaskView_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ViewMode = KS.Gantt.Gantt.ViewModes.Tasks;            
        }

        void mnuResourceView_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ViewMode = KS.Gantt.Gantt.ViewModes.Resources;            
        }               

        void mnuSelectAll_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.SelectAll();            
        }

        void mnuInverseSelection_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.InverseSelection();                       
        }

        void mnuCollapseGroups_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.CollapseAll();
        }

        void mnuExpandGroups_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ExpandAll();
        }

        void mnuUndo_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Undo();
        }

        void mnuRedo_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Redo();            
        }

        void mnuCut_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Cut();
        }

        void mnuCopy_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Copy();
        }

        void mnuPaste_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Paste();
        }

        void mnuDelete_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Delete();
        }        
             
        void gantt1_BeforeDelete(object sender, KS.Gantt.GanttCancelEventArgs e)
        {
            string strMessage = "";

            switch (ucGantt1.gantt1.ViewMode)
            {
                case Gantt.ViewModes.Tasks:
                    strMessage = String.Format(Properties.Resources.Question_DeleteItems, ucGantt1.gantt1.SelectedItems.Count.ToString());
                    break;

                case Gantt.ViewModes.Resources:
                    strMessage = String.Format(Properties.Resources.Question_DeleteItems, ucGantt1.gantt1.SelectedResources.Count.ToString());
                    break;
            }

            if (Program.QuestionMsgBox(strMessage) != DialogResult.Yes)
                e.Cancel = true;            
        }

        void gantt1_ItemDoubleClick(object sender, KS.Gantt.GanttItemMouseEventArgs e)
        {
            ucGantt1.gantt1.EditItem(e.Item);
        }

        void gantt1_ItemsChanged(object sender, EventArgs e)
        {
            UpdateMenu();            
        }

        void gantt1_SelectionChanged(object sender)
        {
            UpdateMenu();            
        }

        void clipChange_ClipboardChanged(object sender, EventArgs e)
        {
            Application.DoEvents();
            UpdateMenu();
        }

        void ganttDataGrid1_UpdateEditMenu(object sender, EventArgs e)
        {
            UpdateMenu();
        }

        void gantt1_ViewModeChanged(object sender, EventArgs e)
        {
            switch (this.ucGantt1.gantt1.ViewMode)
            {
                case Gantt.ViewModes.Tasks:
                    if (tabMain.SelectedTab != tabGantt)
                        tabMain.SelectedTab = tabGantt;
                    break;

                case Gantt.ViewModes.Resources:
                    if (tabMain.SelectedTab != tabResourceView)
                        tabMain.SelectedTab = tabResourceView;
                    break;
            }

            UpdateMenu();
        }

        private void UpdateMenu()
        {            
            mnuUndo.Enabled = ucGantt1.ganttDataGrid1.CanUndo;
            mnuUndo1.Enabled = mnuUndo.Enabled;
            mnuRedo.Enabled = ucGantt1.ganttDataGrid1.CanRedo;
            mnuRedo1.Enabled = mnuRedo.Enabled;
                        
            mnuTaskView.Checked = ucGantt1.gantt1.ViewMode == KS.Gantt.Gantt.ViewModes.Tasks;            
            mnuResourceView.Checked = ucGantt1.gantt1.ViewMode == KS.Gantt.Gantt.ViewModes.Resources;

            mnuSave.Enabled = ucGantt1.gantt1.Modified;
            mnuSave1.Enabled = mnuSave.Enabled;
            
            mnuSaveAs.Enabled = ProjectFilename != "";

            mnuPrint.Enabled = true;

            mnuSolve.Enabled = ucGantt1.gantt1.Tasks.Count > 1;
            mnuSolve1.Enabled = mnuSolve.Enabled;

            bool bGanttVisible = tabMain.SelectedIndex < 2;

            mnuCut.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanCut;
            mnuCopy.Enabled = bGanttVisible & ucGantt1.ganttDataGrid1.CanCopy;
            mnuPaste.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanPaste;
            mnuDelete.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanDelete;
            mnuSelectAll.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanSelectAll;
            mnuInverseSelection.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanInverseSelection;

            mnuZoomIn.Enabled = bGanttVisible;
            mnuZoomOut.Enabled = bGanttVisible;
            mnuZoomToFit.Enabled = bGanttVisible;
            mnuCenterNow.Enabled = bGanttVisible;            

            mnuCollapseGroups.Enabled = bGanttVisible;
            mnuExpandGroups.Enabled = bGanttVisible;

            mnuFind.Enabled = bGanttVisible && ucGantt1.gantt1.CanSearch;
            mnuFind1.Enabled = mnuFind.Enabled;
            mnuFindMore.Enabled = bGanttVisible && ucGantt1.gantt1.CanSearchMore;
            mnuFindMore1.Enabled = mnuFindMore.Enabled;

            bool bTaskviewVisible = tabMain.SelectedIndex < 1;

            mnuASAP.Enabled = bTaskviewVisible && ucGantt1.gantt1.Tasks.Count > 1;
            mnuASAP1.Enabled = mnuASAP.Enabled;

            mnuPathDimming.Checked = ucGantt1.gantt1.PathDimming;            
            if (ucGantt1.gantt1.MaxZoomLevel == Gantt.ZoomLevels.Granularity)
            {
                mnuMaxZoomLevelGranularity.Checked = true;
                mnuMaxZoomLevelMinute.Checked = false;                
            }
            else
            {
                mnuMaxZoomLevelGranularity.Checked = false;
                mnuMaxZoomLevelMinute.Checked = true;
            }

            bool bDBConnected = SqlDBConncted;

            mnuFileMRU.Enabled = !bDBConnected;
            mnuSqlServerDisconnect.Enabled = bDBConnected;
            mnuSqlServerDisconnect1.Enabled = bDBConnected;
            mnuSQLDropDatabase.Enabled = bDBConnected;
            mnuSqlServerDeleteProject.Enabled = bDBConnected && SQLDBOpenFlag;            

            string strText = ucGantt1.gantt1.Project.Text;
            if (ProjectFilename != "")
                strText += " [" + KS.Common.Strings.GetFilename(ProjectFilename) + "]";
            else if (bDBConnected)
                strText += " [" + Program.SqlDB.DBServer + "\\" + Program.SqlDB.DBCatalog + "]";

            strText += " - " + Program.DialogTitle;
            this.Text = strText;            
        }        

        void gantt1_ItemHover(object sender, GanttItemEventArgs e)
        {
            ShowItemStatus(e.Item);
        }

        void gantt1_ItemsResized(object sender, EventArgs e)
        {
            ShowItemStatus(null);
        }

        void gantt1_ItemsMoved(object sender, EventArgs e)
        {
            ShowItemStatus(null);
        }

        void gantt1_MovingItem(object sender, GanttItemEventArgs e)
        {
            ShowItemStatus(e.Item);
        }        

        private bool LastEnableGUI = true;
        public void EnableGUI(bool bEnable)
        {
            if (LastEnableGUI == bEnable)
                return;

            LastEnableGUI = bEnable;

            foreach (Control c in this.Controls)
            {
                c.Enabled = bEnable;
            }

            this.toolStripStatusLabelTaskName.Enabled = true;
            this.toolStripStatusLabelStatus.Enabled = true;
            this.Focus();
        }

        public void ShowStatus()
        {
            ShowStatus("", false);
        }

        public void ShowStatus(string Status, bool bWaitCursor)
        {
            if (Status == "")
            {
                Status = Properties.Resources.StatusReady;
                Program.CursorDefault();
                toolStripStatusLabelStatus.Image = null;
            }
            else
            {
                if (bWaitCursor)
                {
                    Program.CursorWait();
                    toolStripStatusLabelStatus.Image = Properties.Resources.wait;
                }
            }

            toolStripStatusLabelStatus.Text = Status;
            statusStrip1.Update();
        }

        private string LastItemStatusKey = null;
        private void ShowItemStatus(GanttItem item)
        {
            if (!(ucGantt1.gantt1.IsBarMoving || ucGantt1.gantt1.IsBarSizing))
            {
                if ((LastItemStatusKey == null && item == null) || (LastItemStatusKey != null && item != null && item.Key == LastItemStatusKey))
                    return;
            }

            MethodInvoker ShowItemStatusInvoker = delegate
            {
                if (item == null)
                {
                    toolStripStatusLabelTaskName.Text = "";
                    toolStripStatusLabelStartDate.Text = "";
                    toolStripStatusLabelEndDate.Text = "";
                    toolStripStatusLabelDuration.Text = "";
                    LastItemStatusKey = null;
                }
                else
                {
                    toolStripStatusLabelTaskName.Text = item.Text;
                    toolStripStatusLabelStartDate.Text = item.StartDateForDisplay;
                    toolStripStatusLabelEndDate.Text = item.EndDateForDisplay;
                    toolStripStatusLabelDuration.Text = item.DurationForDisplay;
                    LastItemStatusKey = item.Key;
                }
            };

            Invoke(ShowItemStatusInvoker);
        }        

        // *** MRUManager ***
        public void OpenMRUFile(string fileName)
        {
            if (!AskSave())
                return;

            OpenFile(fileName);
        }
       

        // ******* SQL Server *******


        private bool SqlDBConncted
        {
            get
            {
                return Program.SqlDB != null && Program.SqlDB.IsConnected;
            }
        }
       
        void mnuSqlServerConnect_Click(object sender, EventArgs e)
        {
            if (!AskSave())
                return;

            if (Program.SqlDB == null)
            {
                Program.SqlDB = new ProjectPlanner.SqlServer.GanttSqlDB();
                Program.SqlDB.SqlCreateScriptPath = Strings.ApplicationPath(false) + @"\sql\CreateDatabase.sql";
                Program.SqlDB.SqlUpdateScriptPath = Strings.ApplicationPath(false) + @"\sql\UpdateDatabase.sql";
            }

            SqlServer.SqlServerConnectDialog Dlg = new ProjectPlanner.SqlServer.SqlServerConnectDialog();

            Program.SqlDB.LoadSettings();
            Dlg.Server = Program.SqlDB.DBServer;
            //Dlg.Database = Program.SqlDB.DBCatalog;
            Dlg.Password = Program.SqlDB.DBPassword;
            Dlg.UserName = Program.SqlDB.DBUser;
            if (Program.SqlDB.DBWinSecurity)            
                Dlg.SQLAuth = ProjectPlanner.SqlServer.SQLAuthTypes.SQLAuth_Win;            
            else            
                Dlg.SQLAuth = ProjectPlanner.SqlServer.SQLAuthTypes.SQLAuth_SQL;

        LabelRestart:

            Dlg.Database = Program.SqlDB.DBCatalog;
            Dlg.ShowDialog(this);
            this.Update();

            if (Dlg.DialogResult == System.Windows.Forms.DialogResult.OK)
            {
                Program.SqlDB.DBServer = Dlg.Server;
                Program.SqlDB.DBCatalog = Dlg.Database;
                Program.SqlDB.DBWinSecurity = (Dlg.SQLAuth == ProjectPlanner.SqlServer.SQLAuthTypes.SQLAuth_Win);
                if (Program.SqlDB.DBWinSecurity)
                {
                    Program.SqlDB.DBPassword = "";
                    Program.SqlDB.DBUser = "";
                }
                else
                {
                    Program.SqlDB.DBPassword = Dlg.Password;
                    Program.SqlDB.DBUser = Dlg.UserName;
                }
                
                Program.SqlDB.SaveSettings();

                ShowStatus(Properties.Resources.Status_ConnectingSQL, true);

                bool bOK = false;
                
                try
                {
                    bOK = Program.SqlDB.Connect();

                    if (bOK)
                    {

                        ucGantt1.gantt1.ClearData();
                        SetGanttDefaults();
                        ucGantt1.gantt1.ResetModified();
                        ucGantt1.gantt1.HistoryClear();

                        ProjectFilename = "";
                        SQLDBOpenFlag = false;

                        mnuOpen.Image = Properties.Resources.database_get;
                        mnuOpen1.Image = Properties.Resources.database_get;
                        mnuSave.Image = Properties.Resources.database_save;
                        mnuSave1.Image = Properties.Resources.database_save;

                        mruManager.Enabled = false;

                        mnuSqlServerConnect1.Visible = false;
                        mnuSqlServerDisconnect1.Visible = true;

                        if (Program.SqlDB.ProjectCount > 0)
                        {
                            ShowStatus();
                            mnuOpen_Click(null, null);
                        }

                        if (!SQLDBOpenFlag)
                        {
                            ShowStatus(Properties.Resources.Status_ReadingSQL, true);
                            Program.SqlDB.ReadDBGlobal(ucGantt1.gantt1);
                        }
                    }
                }
                catch (Exception ex)
                {
                    Program.ErrMsgBox(ex.Message);
                    bOK = false;
                }
                finally
                {
                    ShowStatus();
                    ShowDBConnectionStatus();
                    UpdateMenu();
                }

                if (!bOK)                
                    goto LabelRestart;                                
            }
            else
            {
                ShowDBConnectionStatus();
                UpdateMenu();
                ShowStatus();
            }

            Dlg.Dispose();
            Dlg = null;
        }

        void mnuSqlServerDisconnect_Click(object sender, EventArgs e)
        {
            SqlServerDisconnect();
        }

        bool SqlServerDisconnect()
        {
            if (!AskSave())
                return false;

            if (Program.SqlDB != null)
            {
                Program.SqlDB.Disconnect();
            }

            ucGantt1.gantt1.ClearData();
            SetGanttDefaults();
            ucGantt1.gantt1.ResetModified();
            ucGantt1.gantt1.HistoryClear();

            mnuOpen.Image = Properties.Resources.folder;
            mnuOpen1.Image = Properties.Resources.folder;
            mnuSave.Image = Properties.Resources.save;
            mnuSave1.Image = Properties.Resources.save;
            mruManager.Enabled = true;

            mnuSqlServerConnect1.Visible = true;
            mnuSqlServerDisconnect1.Visible = false;

            ProjectFilename = "";
            SQLDBOpenFlag = false;

            ShowDBConnectionStatus();
            UpdateMenu();

            return true;
        }

        public void ShowDBConnectionStatus()
        {            
            if (SqlDBConncted)
            {
                toolStripStatusLabelDBConnection.Text = "Connected to " + Program.SqlDB.DBServer + "\\" + Program.SqlDB.DBCatalog + " (" + Program.SqlDB.CurrentUser + ")";
                toolStripStatusLabelDBConnection.Image = Properties.Resources.sqlServer_Connected;
                toolStripStatusLabelDBConnection.AutoSize = true;
                toolStripStatusLabelDBConnection.Visible = true;                
            }
            else
            {                
                toolStripStatusLabelDBConnection.Visible = false;
            }
        }

        void mnuSqlServerDeleteProject_Click(object sender, EventArgs e)
        {
            if (Program.SqlDB == null || !Program.SqlDB.IsConnected)
                return;

            if (Program.QuestionMsgBox(Properties.Resources.Message_DropProject) != DialogResult.Yes)
                return;

            string ProjectKey = ucGantt1.gantt1.Project.Key;

            try
            {
                ShowStatus(Properties.Resources.Status_WritingSQL, true);
                Program.SqlDB.DropProject(ucGantt1.gantt1, ProjectKey);

                ShowStatus(Properties.Resources.Status_ReadingSQL, true);
                Program.SqlDB.ReadDBGlobal(ucGantt1.gantt1);
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {
                ShowDBConnectionStatus();
                ShowStatus();
                UpdateMenu();
            }
        }

        void mnuSQLCreateNewDatabase_Click(object sender, EventArgs e)
        {
            if (SqlDBConncted && !SqlServerDisconnect())
                return;

            SqlServer.CreateNewDatabaseDialog Dlg = new ProjectPlanner.SqlServer.CreateNewDatabaseDialog();

            if (Program.SqlDB == null)
            {
                Program.SqlDB = new ProjectPlanner.SqlServer.GanttSqlDB();
                Program.SqlDB.SqlCreateScriptPath = Strings.ApplicationPath(false) + @"\sql\CreateDatabase.sql";
                Program.SqlDB.SqlUpdateScriptPath = Strings.ApplicationPath(false) + @"\sql\UpdateDatabase.sql";
            }

            Program.SqlDB.LoadSettings();
            Dlg.Server = Program.SqlDB.DBServer;
            // Dlg.Database = Program.SqlDB.DBCatalog; // We ask for a new database
            Dlg.Password = Program.SqlDB.DBPassword;
            Dlg.UserName = Program.SqlDB.DBUser;
            if (Program.SqlDB.DBWinSecurity)
                Dlg.SQLAuth = ProjectPlanner.SqlServer.SQLAuthTypes.SQLAuth_Win;
            else
                Dlg.SQLAuth = ProjectPlanner.SqlServer.SQLAuthTypes.SQLAuth_SQL;

            Dlg.SqlScriptPath = Program.SqlDB.SqlCreateScriptPath;

            Dlg.ShowDialog(this);
            bool bCreatedFlag = Dlg.DatabaseCreated;            

            Dlg.Dispose();
            Dlg = null;

            if (bCreatedFlag)
                mnuSqlServerConnect_Click(null, null);
        }

        void mnuSQLDropDatabase_Click(object sender, EventArgs e)
        {
            if (Program.SqlDB == null || !Program.SqlDB.IsConnected)
                return;

            SqlServer.DropDatabaseDialog Dlg = new ProjectPlanner.SqlServer.DropDatabaseDialog();
            Dlg.GanttDB = Program.SqlDB;

            bool bOK = Dlg.ShowDialog(this) == DialogResult.OK;
            Dlg.Dispose();
            Dlg = null;

            if (bOK)
            {
                Program.SqlDB.DBCatalog = "";
                Program.SqlDB.SaveSettings();
                ucGantt1.gantt1.ClearData();

                mnuSqlServerConnect1.Visible = true;
                mnuSqlServerDisconnect1.Visible = false;

                mnuOpen.Image = Properties.Resources.folder;
                mnuOpen1.Image = Properties.Resources.folder;
                mnuSave.Image = Properties.Resources.save;
                mnuSave1.Image = Properties.Resources.save;
                mruManager.Enabled = true;
            }

            UpdateMenu();
        }

        void mnuFind_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ShowSearchDialog();
            UpdateMenu();            
        }

        void mnuFindMore_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Search();
        }        
        
        private void SetUpContextMenu()
        {
            if (m_ContextMenu != null)
            {
                m_ContextMenu.Dispose();
                m_ContextMenu = null;
            }

            m_ContextMenu = new ContextMenuStrip();

            ToolStripItem cmEdit = m_ContextMenu.Items.Add(mnuEdit.Text + "..");
            cmEdit.Font = new Font(mnuEdit.Font.FontFamily, mnuEdit.Font.Size, FontStyle.Bold);
            cmEdit.Click += new EventHandler(cmEdit_Click);
            m_ContextMenu.Items.Add(new ToolStripSeparator());

            foreach (ToolStripItem item in mnuEdit.DropDownItems)
            {
                if (item is ToolStripSeparator)
                {
                    m_ContextMenu.Items.Add(new ToolStripSeparator());
                }
                else
                {
                    ToolStripItem ti = m_ContextMenu.Items.Add(item.Text, item.Image);
                    ti.Name = item.Name;                    
                    MethodAndSource.CopyEvents(item, ti, "Click");
                }                            
            }

            ucGantt1.gantt1.ContextMenuStrip = m_ContextMenu;
            m_ContextMenu.Opening += new CancelEventHandler(m_ContextMenu_Opening);
        }

        void cmEdit_Click(object sender, EventArgs e)
        {
            if (ucGantt1.gantt1.LastSelectedItem != null)
                ucGantt1.gantt1.EditItem(ucGantt1.gantt1.LastSelectedItem);
        }

        void m_ContextMenu_Opening(object sender, CancelEventArgs e)
        {            
            m_ContextMenu.Items[0].Visible = ucGantt1.gantt1.LastSelectedItem != null;
            m_ContextMenu.Items[1].Visible = ucGantt1.gantt1.LastSelectedItem != null;

            m_ContextMenu.Items["mnuUndo"].Enabled = ucGantt1.gantt1.CanUndo;
            m_ContextMenu.Items["mnuRedo"].Enabled = ucGantt1.gantt1.CanRedo;

            m_ContextMenu.Items["mnuCut"].Enabled = ucGantt1.gantt1.CanCut;
            m_ContextMenu.Items["mnuDelete"].Enabled = ucGantt1.gantt1.CanDelete;
            m_ContextMenu.Items["mnuCopy"].Enabled = ucGantt1.gantt1.CanCopy;
            m_ContextMenu.Items["mnuPaste"].Enabled = ucGantt1.gantt1.CanPaste;

            m_ContextMenu.Items["mnuSelectAll"].Enabled = ucGantt1.gantt1.CanSelectAll;
            m_ContextMenu.Items["mnuInverseSelection"].Enabled = ucGantt1.gantt1.CanSelectAll;

            m_ContextMenu.Items["mnuFind"].Enabled = ucGantt1.gantt1.CanSearch;
            m_ContextMenu.Items["mnuFindMore"].Enabled = ucGantt1.gantt1.CanSearchMore;
        }

        // ************ Testing ***********

#if (DEBUG)

        // The purpose of these tests is to test for memory leaks
        // When there are memory leaks, you'll typically get errors about a handle can't be created
        // Most applications won't pass such a test, so give it a try..
        // The garbage collector in DotNet Framework 4 has it much harder to know, 
        // what's dead and no longer in use, because of all this async/parallel stuff.
        // You'll need a tool such as
        // http://memprofiler.com/
        // to analyze and fix such problems
        //
        // *******************************************************************************
        // The following self-tests prove that the KS-Gantt Control library does not leak.
        // More such test have been performed with the different classes of the library.
        // *******************************************************************************        
        // Run the Self-Test from the Extras menu in Debug-Mode

        void mnuDoSelfTests_Click(object sender, EventArgs e)
        {
            // this self test will run for some hours
            // at minimum, test some thousand iterations for each dialog..
            int iter = 10000;

            // comment out what you don't want to test

            // this adds 100 tasks, 50 groups and 100 dependencies 
            // to a gantt control in a new window and performs a layout of the tasks
            // it also shows, that the KS-Gantt control is highly optimized and really fast
            //SelfTestGanttControlForm(iter);

            //SelfTestProjectSettingsDialog(iter);
            SelfTestTaskEditDialog(iter);
            SelfTestGroupEditDialog(iter);
            SelfTestResourceEditDialog(iter);            

            ShowStatus();
            Program.InfoMsgBox("Self tests passed.");
        }

        private void SelfTestProjectSettingsDialog(int iterations)
        {            
            for (int i = 0; i < iterations; i++)
            {                
                KS.Gantt.Dialogs.ProjectSettingsDialog Dlg = new KS.Gantt.Dialogs.ProjectSettingsDialog();
                Dlg.Edit(ucGantt1.gantt1, ucGantt1.gantt1.Project);
                Dlg.Show(this);

                Application.DoEvents();
                for (int k = 0; k < Dlg.tabMain.TabPages.Count; k++)
                {
                    Dlg.tabMain.SelectedTab = Dlg.tabMain.TabPages[k];
                    Application.DoEvents();                    
                }

                Dlg.Close();
                Dlg.Dispose();
                Dlg = null;

                ShowStatus("ProjectSettings Dialog Iteration " + i.ToString("n0"), false);
            }
        }

        private void SelfTestTaskEditDialog(int iterations)
        {            
            TaskItem task = ucGantt1.gantt1.AddTask(DateTime.Now, DateTime.Now.AddDays(5), "Hello World");

            for (int i = 0; i < iterations; i++)
            {
                KS.Gantt.Dialogs.TaskEditDialog Dlg = new KS.Gantt.Dialogs.TaskEditDialog();
                Dlg.Edit(ucGantt1.gantt1, task);
                Dlg.Show(this);

                Application.DoEvents();
                for (int k = 0; k < Dlg.tabMain.TabPages.Count; k++)
                {
                    Dlg.tabMain.SelectedTab = Dlg.tabMain.TabPages[k];
                    Application.DoEvents();

                    if (k == Dlg.tabMain.TabPages.Count - 2)
                    {
                        foreach (TabPage tp in Dlg.ucBarFormatStyle1.tabMain.TabPages)
                        {
                            Dlg.ucBarFormatStyle1.tabMain.SelectedTab = tp;
                            Application.DoEvents();
                        }
                    }
                }

                Dlg.Close();
                Dlg.Dispose();
                Dlg = null;

                ShowStatus("TaskEdit Dialog Iteration " + i.ToString("n0"), false);
            }            
        }

        private void SelfTestGroupEditDialog(int iterations)
        {
            GroupItem group = ucGantt1.gantt1.AddGroup("Hello World");            
            TaskItem task = ucGantt1.gantt1.AddTask(DateTime.Now, DateTime.Now.AddDays(5), "Hello World");
            group.AddChild(task);

            for (int i = 0; i < iterations; i++)
            {
                KS.Gantt.Dialogs.GroupEditDialog Dlg = new KS.Gantt.Dialogs.GroupEditDialog();
                Dlg.Edit(ucGantt1.gantt1, group);
                Dlg.Show(this);

                Application.DoEvents();
                for (int k = 0; k < Dlg.tabMain.TabPages.Count; k++)
                {
                    Dlg.tabMain.SelectedTab = Dlg.tabMain.TabPages[k];
                    Application.DoEvents();

                    if (k == Dlg.tabMain.TabPages.Count - 2)
                    {
                        foreach (TabPage tp in Dlg.ucGroupFormatStyle1.tabMain.TabPages)
                        {
                            Dlg.ucGroupFormatStyle1.tabMain.SelectedTab = tp;
                            Application.DoEvents();                            
                        }
                    }
                }

                Dlg.Close();
                Dlg.Dispose();
                Dlg = null;

                ShowStatus("GroupEdit Dialog Iteration " + i.ToString("n0"), false);
            }
        }

        private void SelfTestResourceEditDialog(int iterations)
        {
            GroupItem group = ucGantt1.gantt1.AddGroup("Hello World");
            TaskItem task = ucGantt1.gantt1.AddTask(DateTime.Now, DateTime.Now.AddDays(5), "Hello World");
            group.AddChild(task);

            ResourceItem resource = ucGantt1.gantt1.AddResource("Test Resource");
            task.AddResource(resource);

            for (int i = 0; i < iterations; i++)
            {
                KS.Gantt.Dialogs.ResourceEditDialog Dlg = new KS.Gantt.Dialogs.ResourceEditDialog();
                Dlg.Edit(ucGantt1.gantt1, resource);
                Dlg.Show(this);

                Application.DoEvents();
                for (int k = 0; k < Dlg.tabMain.TabPages.Count; k++)
                {
                    Dlg.tabMain.SelectedTab = Dlg.tabMain.TabPages[k];
                    Application.DoEvents();
                }

                Dlg.Close();
                Dlg.Dispose();
                Dlg = null;

                ShowStatus("ResourceEdit Dialog Iteration " + i.ToString("n0"), false);
            }
        }

        private void SelfTestGanttControlForm(int iterations)
        {
            for (int i = 0; i < iterations; i++)
            {
                SelfTestGanttForm Dlg = new SelfTestGanttForm();                
                Dlg.Show(this);
                Application.DoEvents();

                Dlg.ucGantt1.gantt1.SuspendItemLayout();

                KS.Gantt.GanttTesting testing = new GanttTesting(Dlg.ucGantt1.gantt1);
                testing.AddRandomBars(100, DateTime.Now.Year, DateTime.Now.Year + 5, 100);
                testing.AddRandomGroups(50, 20, 20);
                testing.AddRandomRelations(100);

                Dlg.ucGantt1.gantt1.ResumeItemLayout();
                Dlg.ucGantt1.gantt1.Invalidate();
                Dlg.ucGantt1.ganttDataGrid1.Invalidate();
                Dlg.Refresh();
                Application.DoEvents();

                Dlg.ucGantt1.gantt1.AutoMoveItems();
                Application.DoEvents();

                Dlg.Close();
                Dlg.Dispose();
                Dlg = null;

                ShowStatus("ResourceEdit Dialog Iteration " + i.ToString("n0"), false);
            }
        }

        
#endif
    }
}