/*
{*******************************************************************}
{                                                                   }
{       KS-GANTT DotNet Library                                     }
{       Gantt-Charting Control for Windows Forms                    }
{                                                                   }
{       Copyright (c) 2009 - 2014 by Kroll-Software                 }
{       All Rights Reserved                                         }
{                                                                   }
{   This code is provided 'as is',                                  }
{   without warranties of any kind. Use at your own risk.           }                                                              
{                                                                   }
{   As a license holder of the KS-Gantt Control,                    }
{   you may freely redistribute executables based on                }
{   and created with these contents.                                }
{   You can modify the source code to meet your needs.              }
{                                                                   }
{   However, you must not publish, redistribute or transfer         }
{   this source code to 3rd parties under any conditions.           }
{                                                                   } 
{*******************************************************************}
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using KS.Common;
using KS.Gantt;
using ResourcePlanner.Clipboard;

namespace ResourcePlanner
{
    public partial class frmMain : Form, IMRUClient
    {
        private ClipboardChangeNotifier clipChange = null;
        private ContextMenuStrip m_ContextMenu = null;

        MRUManager mruManager = null;

        public frmMain()
        {
            InitializeComponent();

            if (!this.DesignMode)
                this.Font = SystemFonts.IconTitleFont;

            ucGantt1.gantt1.SuspendItemLayout();

            // This is crucial !!
            // This sets the Gantt-Control to the specific Diagram-Type for this application
            ucGantt1.gantt1.DiagramType = Gantt.DiagramTypes.ResourcePlan;            

            ucGantt1.ganttDataGrid1.SelectionMode = GanttDataGrid.GanttDataGridSelectionModes.Cell;

            ucGantt1.gantt1.ItemsChanged += new EventHandler(gantt1_ItemsChanged);
            ucGantt1.gantt1.SelectionChanged += new Gantt.SelectionChangedEventHandler(gantt1_SelectionChanged);

            ucGantt1.ganttDataGrid1.UpdateEditMenu += new EventHandler(ganttDataGrid1_UpdateEditMenu);            
            ucGantt1.gantt1.ItemDoubleClick += new KS.Gantt.Gantt.ItemDoubleClickEventHandler(gantt1_ItemDoubleClick);
            ucGantt1.gantt1.BeforeDelete += new KS.Gantt.Gantt.BeforeDeleteEventHandler(gantt1_BeforeDelete);

            ucGantt1.gantt1.MovingItem += new KS.Gantt.Gantt.MovingItemEventHandler(gantt1_MovingItem);
            ucGantt1.gantt1.ItemsMoved += new EventHandler(gantt1_ItemsMoved);
            ucGantt1.gantt1.ItemsResized += new EventHandler(gantt1_ItemsResized);
            ucGantt1.gantt1.ItemHover += new KS.Gantt.Gantt.ItemHoverEventHandler(gantt1_ItemHover);                       

            ucGantt1.gantt1.ShowWeekends = false;
            ucGantt1.gantt1.ShowWorkingHours = true;            

            SetGanttDefaults();

            mnuSelectAll.Click += new EventHandler(mnuSelectAll_Click);
            mnuInverseSelection.Click += new EventHandler(mnuInverseSelection_Click);

            mnuCollapseGroups.Click += new EventHandler(mnuCollapseGroups_Click);
            mnuExpandGroups.Click += new EventHandler(mnuExpandGroups_Click);

            mnuUndo.Click += new EventHandler(mnuUndo_Click);
            mnuUndo1.Click += new EventHandler(mnuUndo_Click);
            mnuRedo.Click += new EventHandler(mnuRedo_Click);
            mnuRedo1.Click += new EventHandler(mnuRedo_Click);

            mnuCut.Click += new EventHandler(mnuCut_Click);
            mnuCopy.Click += new EventHandler(mnuCopy_Click);
            mnuPaste.Click += new EventHandler(mnuPaste_Click);
            mnuDelete.Click += new EventHandler(mnuDelete_Click);            

            mnuExit.Click += new EventHandler(mnuExit_Click);
            mnuZoomToFit.Click += new EventHandler(mnuZoomToFit_Click);
            mnuCenterNow.Click += new EventHandler(mnuCenterNow_Click);
            mnuZoomIn.Click += new EventHandler(mnuZoomIn_Click);
            mnuZoomOut.Click += new EventHandler(mnuZoomOut_Click);

            mnuAddTask.Click += new EventHandler(mnuAddTask_Click);
            mnuAddTask1.Click += new EventHandler(mnuAddTask_Click);
            mnuAddGroup.Click += new EventHandler(mnuAddGroup_Click);
            mnuAddGroup1.Click += new EventHandler(mnuAddGroup_Click);
            mnuAddResource.Click += new EventHandler(mnuAddResource_Click);
            mnuAddResource1.Click += new EventHandler(mnuAddResource_Click);

            mnuMoveUp.Click += new EventHandler(mnuMoveUp_Click);
            mnuMoveUp1.Click += new EventHandler(mnuMoveUp_Click);
            mnuMoveDown.Click += new EventHandler(mnuMoveDown_Click);
            mnuMoveDown1.Click += new EventHandler(mnuMoveDown_Click);

            mnuNew1.Click += new EventHandler(mnuNew_Click);
            mnuNew.Click += new EventHandler(mnuNew_Click);
            mnuNewInstance.Click += new EventHandler(mnuNewInstance_Click);

            mnuOpen1.Click += new EventHandler(mnuOpen_Click);
            mnuOpen.Click += new EventHandler(mnuOpen_Click);
            mnuSave1.Click += new EventHandler(mnuSave_Click);
            mnuSave.Click += new EventHandler(mnuSave_Click);
            mnuSaveAs.Click += new EventHandler(mnuSaveAs_Click);

            mnuExportAsImage.Click += new EventHandler(mnuExportAsImage_Click);
            mnuExportAsXML.Click += new EventHandler(mnuExportAsXML_Click);

            mnuImportFromXML.Click += new EventHandler(mnuImportFromXML_Click);

            mnuPrintPageSetup.Click += new EventHandler(mnuPrintPageSetup_Click);
            mnuPrintPreview.Click += new EventHandler(mnuPrintPreview_Click);
            mnuPrint.Click += new EventHandler(mnuPrint_Click);

            mnuAbout.Click += new EventHandler(mnuAbout_Click);
            mnuAbout1.Click += new EventHandler(mnuAbout_Click);
            
            mnuKrollSoftwareWebsite.Click += new EventHandler(mnuKrollSoftwareWebsite_Click);

            mnuProjectSettings.Click += new EventHandler(mnuProjectSettings_Click);
            mnuProjectSettings1.Click += new EventHandler(mnuProjectSettings_Click);

            mnuProjectCalendars.Click += new EventHandler(mnuProjectCalendars_Click);
            mnuProjectCalendars1.Click += new EventHandler(mnuProjectCalendars_Click);

            mnuProjectCost.Click += new EventHandler(mnuProjectCost_Click);
            mnuProjectCost1.Click += new EventHandler(mnuProjectCost_Click);                        

            mnuMaxZoomLevelGranularity.Checked = true;
            mnuMaxZoomLevelGranularity.Click += new EventHandler(mnuMaxZoomLevelGranularity_Click);
            mnuMaxZoomLevelMinute.Click += new EventHandler(mnuMaxZoomLevelMinute_Click);
            ucGantt1.gantt1.MaxZoomLevelChanged += new EventHandler(gantt1_MaxZoomLevelChanged);            

            mnuFind.Click += new EventHandler(mnuFind_Click);
            mnuFindMore.Click += new EventHandler(mnuFindMore_Click);
            mnuFind1.Click += new EventHandler(mnuFind_Click);
            mnuFindMore1.Click += new EventHandler(mnuFindMore_Click);                       

            ucGantt1.gantt1.ResetModified();
            ucGantt1.gantt1.HistoryClear();

            ucGantt1.gantt1.ResumeItemLayout();

            ucGantt1.gantt1.HistoryChanged += new EventHandler(gantt1_HistoryChanged);
            ucGantt1.gantt1.ShowStatus += new Gantt.ShowStatusEventHandler(gantt1_ShowStatus);

            //tabMain.SelectedIndexChanged += new EventHandler(tabMain_SelectedIndexChanged);

            SetUpContextMenu();

            // Test Topo
            ucGantt1.gantt1.CustomItemTextDisplay += new Gantt.ItemTextDisplayEventHandler(gantt1_CustomItemTextDisplay);

            this.KeyDown += new KeyEventHandler(frmMain_KeyDown);

            // *************** Load ******************            

            ucTasks1.GanttControl = ucGantt1.gantt1;
            ucResources1.GanttControl = ucGantt1.gantt1;
            ucGroups1.GanttControl = ucGantt1.gantt1;
            ucSkills1.GanttControl = ucGantt1.gantt1;            

            ucGantt1.gantt1.UndoRedoCalled += new EventHandler(gantt1_UndoRedoCalled);            

            // Set some language specific texts            
            ucGantt1.gantt1.WeekPrefix = Properties.Resources.WeekPrefix;

            // Title for new tasks
            ucGantt1.gantt1.NewTaskName = "New Allocation";

            bool bMaxZoomLevelMinute = Program.GetMySetting("MaxZoomLevelMinute", 1).SafeBool();
            if (bMaxZoomLevelMinute)
                ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Minute;
            else
                ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Granularity;            

            KS.Gantt.Helpers.DialogTitle = Program.DialogTitle;

            // MRU FileList
            mruManager = new MRUManager();
            mruManager.Initialize(
             this,                              // owner form
             mnuFileMRU,                        // Recent Files menu item
             Program.C_RegistrySectionKey);     // Registry path to keep MRU list

            // Optional. Call these functions to change default values:
            //mruManager.CurrentDir = ".....";           // default is current directory
            mruManager.MaxMRULength = 7;             // default is 10
            mruManager.MaxDisplayNameLength = 60;     // default is 40

            // Some Shortcuts
            // Setting Shortcut Keys by resource, throws errors in different Visual-Studio language versions.
            // This is, why we set them by code

            mnuNew.ShortcutKeys = Keys.N | Keys.Control;
            mnuNewInstance.ShortcutKeys = Keys.N | Keys.Control | Keys.Alt;
            mnuOpen.ShortcutKeys = Keys.O | Keys.Control;
            mnuSave.ShortcutKeys = Keys.S | Keys.Control;
            mnuSaveAs.ShortcutKeys = Keys.S | Keys.Control | Keys.Alt;
            mnuPrint.ShortcutKeys = Keys.P | Keys.Control;            

            mnuZoomToFit.ShortcutKeys = Keys.M | Keys.Control;
            mnuCenterNow.ShortcutKeys = Keys.M | Keys.Alt;

            mnuSelectAll.ShortcutKeys = Keys.A | Keys.Control;

            mnuUndo.ShortcutKeys = Keys.Z | Keys.Control;
            mnuRedo.ShortcutKeys = Keys.Y | Keys.Control;

            mnuDelete.ShortcutKeys = Keys.Delete;
            mnuCut.ShortcutKeys = Keys.X | Keys.Control;
            mnuCopy.ShortcutKeys = Keys.C | Keys.Control;
            mnuPaste.ShortcutKeys = Keys.V | Keys.Control;

            mnuFind.ShortcutKeys = Keys.F | Keys.Control;            

            mnuExpandGroups.ShortcutKeys = Keys.Oemplus | Keys.Control;
            mnuCollapseGroups.ShortcutKeys = Keys.OemMinus | Keys.Control;

            ucGantt1.ganttDataGrid1.GotFocus += new EventHandler(ganttDataGrid1_GotFocus);
            ucGantt1.ganttDataGrid1.LostFocus += new EventHandler(ganttDataGrid1_LostFocus);
            ucGantt1.ganttDataGrid1.SelectedRowIndexChanged += new EventHandler(ganttDataGrid1_SelectedRowIndexChanged);


            // The default LayoutLogic.ItemComparer sorts items by Group.RowPosition
            // You can use a custom ItemComparer or use a prebuild like this..
            //ucGantt1.gantt1.LayoutLogic.ItemComparer = new KS.Gantt.LayoutLogic.ResourcePlanItemComparerAlphabetGroups();

            ucGantt1.gantt1.DefaultBarFormatStyle.ForegroundStyle.FontAlign = ContentAlignment.MiddleCenter;

            // Hide the RowHeaders
            // ucGantt1.ganttDataGrid1.RowHeaderWidth = 0;

            // Disallow selection of groups
            //ucGantt1.gantt1.ItemSelected += new Gantt.ItemSelectedEventHandler(gantt1_ItemSelected);

            // Disable the task overlapping check, if you don't need it
            // ((KS.Gantt.LayoutLogic.LayoutLogicResourceTable)ucGantt1.gantt1.LayoutLogic).OverlappingCheckEnabled = false;            

            // Show the Resource Calendar on Datagrid Row-Selection, if you like to do so..            
            ucGantt1.ganttDataGrid1.ShowItemCalendarOnRowChange = true;

            // Setup some colors for the GanttDataGrid
            if (true)   // set false to see the Gantt-Control's default colors
            {
                ucGantt1.ganttDataGrid1.HeaderBackground.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.ganttDataGrid1.HeaderBackground.SetDefault();

                ucGantt1.ganttDataGrid1.HeaderForeground.Color = ucGantt1.gantt1.HeaderManager.MonthHeader.FormatStyle.ForegroundStyle.Color;
                ucGantt1.ganttDataGrid1.HeaderForeground.SetDefault();

                ucGantt1.gantt1.HeaderBackgroundStyle.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.gantt1.HeaderBackgroundStyle.SetDefault();

                ucGantt1.gantt1.HeaderManager.YearHeader.FormatStyle.BackgroundStyle.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.gantt1.HeaderManager.YearHeader.FormatStyle.SetDefault();

                ucGantt1.gantt1.HeaderManager.MonthHeader.FormatStyle.BackgroundStyle.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.gantt1.HeaderManager.MonthHeader.FormatStyle.SetDefault();

                ucGantt1.gantt1.HeaderManager.WeekHeader.FormatStyle.BackgroundStyle.Color = Color.FromArgb(206, 212, 223);
                ucGantt1.gantt1.HeaderManager.WeekHeader.FormatStyle.SetDefault();
            }

            // do this at last
            ucGantt1.gantt1.ResetModified();
            ucGantt1.gantt1.HistoryClear();

            // Add some sample resources
            AddSampleResources();
        }

        void gantt1_ItemSelected(object sender, GanttItemCancelEventArgs e)
        {
            if (e.Item.ItemType == GanttItemType.GroupItem)
                e.Cancel = true;
        }        

        void AddSampleResources()
        {
            ucGantt1.gantt1.SuspendItemLayout();
            ucGantt1.ganttDataGrid1.SuspendItemLayout();

            for (int i = 1; i <= 16; i++)
            {
                ResourceItem resource = ucGantt1.gantt1.AddResource("Room " + i.ToString());
                resource.ResourceType = ResourceTypes.Material;
            }

            // Add some weired styles to some resources..
            // Not, that it's looking good, just to show how it works..

            GlossyBackgroundStyle bgs;
            ForegroundStyle fgs;

            bgs = new GlossyBackgroundStyle();            
            bgs.Color = Color.Red;
            bgs.GlossGradient = false;  // make it flat
            ucGantt1.gantt1.Resources[2].DataGridRowBackgroundStyle = bgs;

            bgs = new GlossyBackgroundStyle();
            bgs.Color = Color.FromArgb(80, Color.Red);
            bgs.GlossGradient = false;
            ucGantt1.gantt1.Resources[2].RowBackgroundStyle= bgs;

            fgs = new ForegroundStyle();
            fgs.Font = new Font(this.Font.FontFamily, this.Font.Size, FontStyle.Bold);
            fgs.Color = Color.Yellow;
            ucGantt1.gantt1.Resources[2].DataGridRowForegroundStyle = fgs;            

            // ---

            bgs = new GlossyBackgroundStyle();
            bgs.Color = Color.FromArgb(196, Color.DeepSkyBlue);
            bgs.GlossGradient = true;            
            ucGantt1.gantt1.Resources[12].DataGridRowBackgroundStyle = bgs;

            bgs = new GlossyBackgroundStyle();
            bgs.Color = Color.FromArgb(80, Color.DeepSkyBlue);
            ucGantt1.gantt1.Resources[12].RowBackgroundStyle = bgs;

            fgs = new ForegroundStyle();
            fgs.Font = new Font("Times", this.Font.Size + 2, FontStyle.Bold | FontStyle.Italic);
            fgs.Color = Color.DarkMagenta;
            ucGantt1.gantt1.Resources[12].DataGridRowForegroundStyle = fgs;            
            
            // ---

            bgs = new GlossyBackgroundStyle();
            bgs.Color = Color.FromArgb(196, Color.Yellow);
            bgs.ColorEffect = Color.FromArgb(196, Color.DarkViolet);
            bgs.Hatch = true;
            bgs.HatchStyle = System.Drawing.Drawing2D.HatchStyle.ForwardDiagonal;
            ucGantt1.gantt1.Resources[14].DataGridRowBackgroundStyle = bgs;

            bgs = new GlossyBackgroundStyle();
            bgs.Color = Color.FromArgb(80, Color.Yellow);
            bgs.ColorEffect = Color.FromArgb(80, Color.DarkViolet);
            bgs.Hatch = true;
            bgs.GlossGradient = true;   // this is the default
            bgs.HatchStyle = System.Drawing.Drawing2D.HatchStyle.ForwardDiagonal;
            ucGantt1.gantt1.Resources[14].RowBackgroundStyle = bgs;

            fgs = new ForegroundStyle();
            fgs.Font = new Font("Impact", this.Font.Size + 5, FontStyle.Regular | FontStyle.Strikeout);
            fgs.Color = Color.DarkViolet;
            ucGantt1.gantt1.Resources[14].DataGridRowForegroundStyle = fgs;

            // You don't wanna show the 'Role' Column in the Datagrid?
            // Remove it!
            // here are some samples for the columns
            //
            // Set visible = false
            ucGantt1.ganttDataGrid1.ResourceTableColumns["Role"].Visible = false;
            // Or remove the column
            //ucGantt1.ganttDataGrid1.ResourceTableColumns.Remove(ucGantt1.ganttDataGrid1.ResourceTableColumns["Role"]);
                        
            //  ucGantt1.ganttDataGrid1.ResourceTableColumns.Remove(ucGantt1.ganttDataGrid1.ResourceTableColumns["Icons"]);
            //  ucGantt1.ganttDataGrid1.ResourceTableColumns["Resource"].Width = 360;
            //  ucGantt1.ganttDataGrid1.ResourceTableColumns["Resource"].AutoMinWidth = true;
            //  ucGantt1.ganttDataGrid1.ResourceTableColumns["Resource"].AllowResize = true;                        
            
            
            // Every resource can have it's custom calendar
            // In this sample we let resource 3 not be available on Tuesday and Thursday

            // first initialize a calendar for this resource
            if (ucGantt1.gantt1.Resources[2].Calendar == null)
            {
                KS.Gantt.Calendars.Calendar calendar = new KS.Gantt.Calendars.Calendar();
                calendar.InheritedCalendarKey = ucGantt1.gantt1.DefaultCalendar.Key;

                // This is a resource calendar. This must be set.
                calendar.IsResourceCalendar = true;

                calendar.InitDefaultWorkingDays();
                ucGantt1.gantt1.CalendarList.Add(calendar);

                ucGantt1.gantt1.Resources[2].CalendarKey = calendar.Key;
            }

            ucGantt1.gantt1.Resources[2].Calendar.SetWorkingDay(DayOfWeek.Tuesday, false);
            ucGantt1.gantt1.Resources[2].Calendar.SetWorkingDay(DayOfWeek.Thursday, false);

            // December 25 is a holiday, set it
            ucGantt1.gantt1.Resources[2].Calendar.AddException(new KS.Gantt.Calendars.CalendarException(new DateTime(2011, 12, 25), false));

            // On December 24, the resource is available until 14:00h
            KS.Gantt.Calendars.CalendarException exception = new KS.Gantt.Calendars.CalendarException(new DateTime(2011, 12, 24), false);            
            exception.AddWorkingHour(new KS.Gantt.Calendars.WorkingHour(8, 14));
            ucGantt1.gantt1.Resources[2].Calendar.AddException(exception);            

            // Now let's add some sample tasks
            TaskItem task;
            
            task = new TaskItem(ucGantt1.gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(-10);
            task.Duration = TimeSpan.FromDays(14);

            // We MUST add one resource in DiagramType = ResourceTable !
            // Or set the appropriate RowIndex
            task.AddResource(ucGantt1.gantt1.Resources[2]);
            task.Text = "Sample Allocation 1";            
            ucGantt1.gantt1.AddTask(task);

            task = new TaskItem(ucGantt1.gantt1.Granularity);
            task.TaskType = TaskItem.TaskTypes.FixedUnits;
            task.StartDate = DateTime.Now.AddDays(6);
            task.Duration = TimeSpan.FromDays(12);

            // We MUST add one resource in DiagramType = ResourceTable !
            // Or set the appropriate RowIndex
            task.AddResource(ucGantt1.gantt1.Resources[4]);
            task.Text = "Sample Allocation 2";            
            ucGantt1.gantt1.AddTask(task);

            // Let's add two more tasks on another row with a dependency

            TaskItem Task1;
            Task1 = new TaskItem(ucGantt1.gantt1.Granularity);
            Task1.TaskType = TaskItem.TaskTypes.FixedUnits;
            Task1.StartDate = DateTime.Now.AddDays(-10);
            Task1.Duration = TimeSpan.FromDays(10);
            Task1.AddResource(ucGantt1.gantt1.Resources[7]);
            Task1.Text = "Sample Allocation 3";
            Task1.LockToRow = true;
            ucGantt1.gantt1.AddTask(Task1);

            TaskItem Task2;
            Task2 = new TaskItem(ucGantt1.gantt1.Granularity);
            Task2.TaskType = TaskItem.TaskTypes.FixedUnits;
            Task2.StartDate = DateTime.Now.AddDays(-10);
            Task2.Duration = TimeSpan.FromDays(5);
            Task2.AddResource(ucGantt1.gantt1.Resources[7]);
            Task2.Text = "Sample Allocation 4";
            ucGantt1.gantt1.AddTask(Task2);

            // Add the dependency
            Task2.Dependencies.Add(new TaskRelation(Task1));

            // Dependency Options: End-Start, 1 Day delay
            Task2.Dependencies[0].TaskRelationType = TaskRelationTypes.EndStart;
            Task2.Dependencies[0].DelayDays = 1.0;

            // disable showing the critical path (longest path through the project) in a different color
            ucGantt1.gantt1.ShowCriticalPath = false;

            // Set Task2 to be always 'As Soon As Possible', 
            // that means: Task2 will move back, when Task1 is moved backwards
            Task2.ASAP = true;

            //Task2.Visible = false;
                       
            
            // After the task was added to the control, recalculate it's parameters as EndDate..
            // task.Recalculate(true);
            // Or later call it for all tasks..
            //ucGantt1.gantt1.RecalculateTasks();


            // Add some Groups to the control
            GroupItem group;
            
            group = ucGantt1.gantt1.AddGroup("Group 1");
            group.AddChild(ucGantt1.gantt1.Resources[3]);
            group.AddChild(ucGantt1.gantt1.Resources[4]);
            //group.RowPosition = 5;                        

            group = ucGantt1.gantt1.AddGroup("Group 2");
            group.AddChild(ucGantt1.gantt1.Resources[6]);
            group.AddChild(ucGantt1.gantt1.Resources[7]);
            group.AddChild(ucGantt1.gantt1.Resources[8]);
            //group.RowPosition = 6;

            //// Set Default Styles for all groups
            //foreach (GroupItem g in ucGantt1.gantt1.Groups)
            //{
            //    g.FormatStyle.ForegroundStyle.Font = new Font(group.FormatStyle.ForegroundStyle.Font.FontFamily, group.FormatStyle.ForegroundStyle.Font.Size, FontStyle.Bold);
            //    g.FormatStyle.ForegroundStyle.FontAlign = ContentAlignment.MiddleCenter;
            //    g.FormatStyle.BackgroundStyle.Color = Color.FromArgb(180, SystemColors.ActiveCaption);
            //    g.FormatStyle.BackgroundStyle.GlossGradient = true;
            //    g.FormatStyle.BackgroundStyle.GlossBlendStyle = GradientBlendStyle.Blend25;
                
            //    g.DataGridRowBackgroundStyle = (GlossyBackgroundStyle)g.FormatStyle.BackgroundStyle.Clone();
            //}

            // Items are sorted by RowPosition
            // So set RowPosition for all Resources to their index position
            // GroupItems with the same RowPosition are sorted before ResourceItems
            // Items with the same Type and RowPosition are sorted alphabetically
            //foreach (ResourceItem resource in ucGantt1.gantt1.Resources)
            //    resource.RowPosition = ucGantt1.gantt1.Resources.IndexOf(resource);

            // in ucGantt we added an ImageList to the Gantt-Control.
            // lets set some Icons
            Random rnd = new Random(unchecked((int)DateTime.Now.Ticks));
            foreach (ResourceItem resource in ucGantt1.gantt1.Resources)
            {
                resource.ImageIndex = rnd.Next(0, ucGantt1.imageList1.Images.Count);

                if (rnd.NextDouble() > 0.5)
                {
                    for (int i = 0; i < rnd.Next(1, 4); i++)
                    {
                        Image img = ucGantt1.imageList2.Images[rnd.Next(0, ucGantt1.imageList2.Images.Count)];
                        resource.Icons.Add(img);
                    }
                }
            }

            ucGantt1.gantt1.ShowCustomIcons = true;            

            // when you are using the Gantt-Control without a GanttDataGrid, 
            // you can display the Icons on the Gantt-Control.
            // In this case, you must disable all Task-Icons, just show the custom icons
            
            //ucGantt1.gantt1.ShowIcons = true;
            //ucGantt1.gantt1.ShowOverDueIcon = false;
            //ucGantt1.gantt1.ShowStatusIcons = false;
            //ucGantt1.gantt1.ShowPriorityIcons = false;
            //ucGantt1.gantt1.ShowHappinessIcon = false;


            ucGantt1.ganttDataGrid1.ResumeItemLayout();
            ucGantt1.gantt1.ResumeItemLayout();            

            // We added some data, so reset modified states
            ucGantt1.gantt1.ResetModified();
            ucGantt1.gantt1.HistoryClear();
        }               

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.Text = Program.DialogTitle;

            if (Program.GetMySetting("MainMaximized", 0).SafeBool())
            {
                this.WindowState = FormWindowState.Maximized;
            }
            else
            {
                this.Left = Program.GetMySetting("MainLeft", 50).SafeInt(50);
                this.Top = Program.GetMySetting("MainTop", 50).SafeInt(50);
                this.Width = Program.GetMySetting("MainWidth", 800).SafeInt(800);
                this.Height = Program.GetMySetting("MainHeight", 600).SafeInt(600);
            }

            ucGantt1.splitContainer1.SplitterDistance = Program.GetMySetting("GanttSplitterWidth", 190).SafeInt(190);

            if (this.Width < 500)
                this.Width = 500;

            if (this.Height < 400)
                this.Height = 400;

            // SetUp the Clipboard Monitor
            this.clipChange = new ClipboardChangeNotifier();
            this.clipChange.ClipboardChanged += new EventHandler(clipChange_ClipboardChanged);
            this.clipChange.AssignHandle(this.Handle);
            this.clipChange.Install();

            UpdateMenu();
            ShowStatus();            
        }

        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);

            // Open File from the Command Line
            string[] args = Environment.GetCommandLineArgs();
            string FName = "";
            if (args.Length >= 2)
                FName = args[1];

            if (FName != "" && KS.Common.Strings.FileExists(FName))
            {
                Application.DoEvents();
                OpenFile(FName);
            }
        }

        void gantt1_ShowStatus(object sender, GanttStatusEventArgs e)
        {
            if (e.Level == 0)
            {
                ShowStatus(e.StatusText, e.ShowWaitCursor);

                if (bEscPressed)
                    e.Cancel = true;
            }
            else
            {
                toolStripStatusLabelTaskName.Text = e.StatusText;
                statusStrip1.Update();
            }            
        }        
        
        // *** Max Zoom Level ***
        void mnuMaxZoomLevelGranularity_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Granularity;
        }

        void mnuMaxZoomLevelMinute_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.MaxZoomLevel = Gantt.ZoomLevels.Minute;
        }

        void gantt1_MaxZoomLevelChanged(object sender, EventArgs e)
        {
            UpdateMenu();
            if (ucGantt1.gantt1.MaxZoomLevel == Gantt.ZoomLevels.Granularity)
                Program.SaveMySetting("MaxZoomLevelMinute", 0);
            else
                Program.SaveMySetting("MaxZoomLevelMinute", 1);
        }

        private bool GanttVisible
        {
            get
            {
                return tabMain.SelectedIndex < 2;
            }
        }

        private bool bEscPressed = false;
        void frmMain_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Delete:
                    if (GanttVisible)
                    {
                        if (e.Shift || e.Control)
                            ucGantt1.ganttDataGrid1.Cut();
                        else
                            ucGantt1.ganttDataGrid1.Delete();

                        e.Handled = true;
                    }
                    break;

                case Keys.Insert:
                    if (GanttVisible)
                    {
                        if (e.Control)
                        {
                            ucGantt1.ganttDataGrid1.Copy();
                            e.Handled = true;
                        }
                        else if (e.Shift)
                        {
                            ucGantt1.ganttDataGrid1.Paste();
                            e.Handled = true;
                        }
                    }
                    break;

                case Keys.Escape:
                    bEscPressed = true;
                    break;
            }
        }

        void mnuNewInstance_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process instance = null;

            try
            {
                Program.CursorWait();
                instance = new System.Diagnostics.Process();
                instance.StartInfo.FileName = Application.ExecutablePath;
                instance.StartInfo.Arguments = "nosplash";
                instance.Start();
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {
                if (instance != null)
                    instance.Dispose();

                Program.CursorDefault();
            }
        }

        void gantt1_CustomItemTextDisplay(object sender, GanttItemTextEventArgs e)
        {
            bool ShowKeys = false;

            if (e.Item.ItemType == GanttItemType.TaskItem)
            {
                e.Text = e.Item.Text;

                if (ShowKeys)
                    e.Text += " (" + e.Item.Key + ")";

                e.Text += " - Topo: " + ((TaskItem)e.Item).topo_index.ToString();
                e.Text += " - Path: " + ((TaskItem)e.Item).connectedcomponent_index.ToString();
                //e.Text += " - Distance: " + ((TaskItem)e.Item).cpm_Distance.ToString("n2");                

                if (ShowKeys)
                {
                    if (e.Item.ParentItemKey != null)
                        e.Text += " - ParentItemKey: " + e.Item.ParentItemKey;
                    else if (e.Item.ParentItem != null)
                        e.Text += " - ParentItem.Key: " + e.Item.ParentItem.Key;
                }

            }
            else if (e.Item.ItemType == GanttItemType.GroupItem)
            {
                e.Text = e.Item.Text;

                if (ShowKeys)
                    e.Text += " (" + e.Item.Key + ")";
            }
        }

        void gantt1_HistoryChanged(object sender, EventArgs e)
        {
            if (!ucGantt1.gantt1.IsItemLayoutSuspended)
                UpdateMenu();
        }

        void mnuProjectCost_Click(object sender, EventArgs e)
        {
            // We could simply call gantt.CalcProjectCost()
            // but we want to consider selected items, if there are any
            // and also consider the selected item in ResourceView            

            double cost = 0.0;

            bool bSelected = false;

            KS.Gantt.Gantt gantt = ucGantt1.gantt1;
            if (gantt.ViewMode == Gantt.ViewModes.Tasks)
            {
                // Filter selected tasks, if there are any, otherwise calc on all tasks
                bSelected = gantt.SelectedItems.Count > 0;
                cost = gantt.CalcProjectCost(bSelected);
            }
            else
            {
                // SelectedItems is only valid for the TaskView
                // in ResourceView there might be a single 'LastSelectedItem'
                //
                // In ResourceView, there is an extra TaskItemCollection called 'ResourceBars'
                // They are constructed of 'ResourceHeaders' and of simple TaskItems, that are clones of the original tasks
                // A ResourceHearder's Parent is the depending ResourceItem

                GanttItem SelectedItem = gantt.LastSelectedItem;
                if (SelectedItem == null)
                {
                    // calc cost of the entire project 
                    cost = gantt.CalcProjectCost();
                }
                else if (SelectedItem is TaskItem)
                {
                    bSelected = true;

                    TaskItem task = (TaskItem)SelectedItem;
                    if (task.IsResourceHeaderItem && task.ParentItem != null && task.ParentItem.ItemType == GanttItemType.ResourceItem)
                        cost = gantt.CalcProjectCost((ResourceItem)task.ParentItem);
                    else
                        cost = gantt.CalcProjectCost("", task.Key, false);
                }
            }

            string strMessage = Properties.Resources.ProjectCost + ":\r\n";
            if (bSelected)
                strMessage += Properties.Resources.SelectedElements + "\r\n";
            strMessage += "\r\n" + cost.ToString("c");

            Program.InfoMsgBox(strMessage);
        }        
        
        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            EnableGUI(true);
            bEscPressed = true;

            if (!AskSave())
            {
                e.Cancel = true;
                return;
            }

            if ((this.WindowState != FormWindowState.Minimized) && (this.WindowState != FormWindowState.Maximized))
            {
                Program.SaveMySetting("MainLeft", this.Left);
                Program.SaveMySetting("MainTop", this.Top);
                Program.SaveMySetting("MainWidth", this.Width);
                Program.SaveMySetting("MainHeight", this.Height);
            }

            Program.SaveMySetting("GanttSplitterWidth", ucGantt1.splitContainer1.SplitterDistance);

            if (this.WindowState == FormWindowState.Maximized)
                Program.SaveMySetting("MainMaximized", 1);
            else
                Program.SaveMySetting("MainMaximized", 0);

            this.Hide();
        }

        private void mnuAbout_Click(object sender, EventArgs e)
        {
            frmAbout Dlg = new frmAbout();
            Dlg.ShowDialog(this);
            Dlg.Dispose();
        }
        
        private void mnuKrollSoftwareWebsite_Click(object sender, EventArgs e)
        {
            Program.OpenURL(Properties.Resources.KrollWebsiteUrl);
        }

        bool EndEdit()
        {
            if (!ucTasks1.EndEdit())
            {
                tabMain.SelectedTab = tabTasks;
                return false;
            }

            if (!ucResources1.EndEdit())
            {
                tabMain.SelectedTab = tabResources;
                return false;
            }

            if (!ucGroups1.EndEdit())
            {
                tabMain.SelectedTab = tabGroups;
                return false;
            }

            if (!ucSkills1.EndEdit())
            {
                tabMain.SelectedTab = tabSkills;
                return false;
            }

            if (!ucGantt1.ganttDataGrid1.EndEdit(true))
            {
                tabMain.SelectedTab = tabPlan;
                return false;
            }

            return true;
        }


        private void ResetAllBindings()
        {
            ucTasks1.InitData();
            ucResources1.InitData();
            ucGroups1.InitData();
            ucSkills1.InitData();            
        }

        void gantt1_UndoRedoCalled(object sender, EventArgs e)
        {
            ResetAllBindings();
        }

        private void SetGanttDefaults()
        {
            // Gantt-Defaults
            ucGantt1.gantt1.SetDefaultStyles();
            ucGantt1.gantt1.WeekendFormatStyle.BackgroundStyle.Gradient = false;

            ucGantt1.gantt1.DefaultDurationType = TaskItem.DurationTypes.FullWorkingDays;
            ucGantt1.gantt1.DefaultTaskType = TaskItem.TaskTypes.FixedUnits;
            ucGantt1.gantt1.Granularity = Granularities.QuarterHour;
            ucGantt1.gantt1.ResourceViewFilter = Gantt.ResourceViewFilters.AssignedResources;

            // Show Horizontal Lines
            ucGantt1.gantt1.GridLines = true;
            ucGantt1.gantt1.RandomBarColors = true;            
            
            // Show Text inside the bars. Set to none, if you don't want the text
            ucGantt1.gantt1.ItemTextDisplay = Gantt.ItemTextDisplayModes.Text;            

            // Disable dependencies between tasks.
            // This can be enabled as well..
            //ucGantt1.gantt1.AllowDependencies = false;
            ucGantt1.gantt1.AllowDependencies = true;

            // Row and Item Heights
            ucGantt1.gantt1.RowHeight = 28; // Height of the rows
            ucGantt1.gantt1.BarHeight = 20; // Height of the bars            

            // Initial date range
            ucGantt1.gantt1.SuspendItemLayout();
            ucGantt1.gantt1.StartDate = DateTime.Now.AddMonths(-1).AddDays(2);
            ucGantt1.gantt1.EndDate = DateTime.Now.AddMonths(1).AddDays(-2);
            ucGantt1.gantt1.ResumeItemLayout();

            ucGantt1.gantt1.DiagramType = Gantt.DiagramTypes.ResourcePlan;
            ucGantt1.gantt1.ShowIcons = false;

            ucGantt1.ganttDataGrid1.ResourceTableColumns["Role"].Visible = false;
        }

        void mnuNew_Click(object sender, EventArgs e)
        {
            if (tabMain.SelectedIndex > 1)
                tabMain.SelectedTab = tabPlan;

            if (!AskSave())
                return;

            ucGantt1.gantt1.SuspendItemLayout();

            ucGantt1.gantt1.ClearData();
            SetGanttDefaults();            

            ucGantt1.gantt1.HistoryClear();
            ucGantt1.gantt1.ResetModified();

            ucGantt1.gantt1.ResumeItemLayout();

            tabMain.SelectedTab = tabPlan;
            ResetAllBindings();

            ProjectFilename = "";
            UpdateMenu();
        }

        private string ProjectFilename = "";        
        void mnuOpen_Click(object sender, EventArgs e)
        {
            if (!AskSave())
                return;
            
            string FName;

            System.Windows.Forms.OpenFileDialog FileDialog = new System.Windows.Forms.OpenFileDialog();
            FileDialog.Title = Properties.Resources.Caption_Open;
            FileDialog.Filter = Properties.Resources.FileDialog_RPP_Filter;
            FileDialog.CheckFileExists = true;
            FileDialog.Multiselect = false;
            FileDialog.DereferenceLinks = true;
            FileDialog.FilterIndex = 1;
            FileDialog.FileName = "";
            if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
            {
                FileDialog.Dispose();
                return;
            }

            FName = FileDialog.FileName;
            FileDialog.Dispose();

            OpenFile(FName);            

            ucGantt1.gantt1.ShowIcons = false;
        }

        private void OpenFile(string FName)
        {
            tabMain.SelectedTab = tabPlan;
            this.Update();

            try
            {
                ShowStatus(Properties.Resources.Status_ReadingFile, true);
                ucGantt1.gantt1.DeSerializeData(FName);
                ProjectFilename = FName;
                mruManager.Add(FName);
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
                mruManager.Remove(FName);
            }
            finally
            {
                ResetAllBindings();

                ucGantt1.gantt1.HistoryClear();
                ucGantt1.gantt1.ResetModified();

                ShowStatus();
                UpdateMenu();
            }
        }

        bool AskSave()
        {
            if (!EndEdit())
                return false;

            if (!ucGantt1.gantt1.Modified)
                return true;

            switch (Program.QuestionMsgBox(Properties.Resources.Message_SaveChanges))
            {
                case DialogResult.Cancel:
                    return false;

                case DialogResult.No:
                    return true;

                case DialogResult.Yes:
                    return SaveProject(false);
            }

            throw new Exception("Unknown DialogResult");
        }

        public bool SaveProject(bool bSaveAs)
        {
            string FName = "";

            if (bSaveAs || ProjectFilename == "")
            {
                System.Windows.Forms.SaveFileDialog FileDialog = new System.Windows.Forms.SaveFileDialog();

                if (bSaveAs)
                    FileDialog.Title = Properties.Resources.Caption_SaveAs;
                else
                    FileDialog.Title = Properties.Resources.Caption_Save;

                FileDialog.Filter = Properties.Resources.FileDialog_RPP_Filter;
                FileDialog.FilterIndex = 1;
                FileDialog.DefaultExt = ".ppp";
                FileDialog.FileName = KS.Common.Strings.GetFilename(ProjectFilename);
                if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
                {
                    FileDialog.Dispose();
                    return false;
                }

                FName = FileDialog.FileName;
                FileDialog.Dispose();
                this.Update();

                try
                {
                    ShowStatus(Properties.Resources.Status_WritingFile, true);
                    ucGantt1.gantt1.SerializeData(FName);
                    ucGantt1.gantt1.SetHistoryModified();
                    ucGantt1.gantt1.ResetModified();
                    ProjectFilename = FName;
                }
                catch (Exception ex)
                {
                    Program.CursorDefault();
                    Program.ErrMsgBox(ex.Message);
                    return false;
                }
                finally
                {
                    ShowStatus();
                    UpdateMenu();
                }
            }
            else
            {
                try
                {
                    ShowStatus(Properties.Resources.Status_WritingFile, true);
                    ucGantt1.gantt1.SerializeData(ProjectFilename);
                    ucGantt1.gantt1.SetHistoryModified();
                    ucGantt1.gantt1.ResetModified();
                }
                catch (Exception ex)
                {
                    Program.CursorDefault();
                    Program.ErrMsgBox(ex.Message);
                    return false;
                }
                finally
                {
                    ShowStatus();
                    UpdateMenu();
                }
            }

            if (FName != "")
                mruManager.Add(FName);
        

            return true;
        }

        void mnuSave_Click(object sender, EventArgs e)
        {
            SaveProject(false);
        }

        void mnuSaveAs_Click(object sender, EventArgs e)
        {
            SaveProject(true);
        }

        void mnuExportAsImage_Click(object sender, EventArgs e)
        {
            string FName = "";

            System.Windows.Forms.SaveFileDialog FileDialog = new System.Windows.Forms.SaveFileDialog();

            FileDialog.Title = Properties.Resources.Caption_ExportAsImage;

            FileDialog.Filter = Properties.Resources.ExportAsImage_Filter;
            FileDialog.FilterIndex = 1;
            FileDialog.DefaultExt = ".png";
            FileDialog.FileName = "";
            if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
            {
                FileDialog.Dispose();
                return;
            }

            FName = FileDialog.FileName;
            FileDialog.Dispose();
            this.Update();

            Bitmap bmp = null;

            try
            {
                bmp = ucGantt1.ganttDataGrid1.GetBitmap();

                if (bmp != null)
                {
                    bmp.Save(FName, System.Drawing.Imaging.ImageFormat.Png);
                }
            }
            catch (Exception ex)
            {
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {
                if (bmp != null)
                    bmp.Dispose();
            }
        }

        void mnuExportAsXML_Click(object sender, EventArgs e)
        {
            string FName = "";

            System.Windows.Forms.SaveFileDialog FileDialog = new System.Windows.Forms.SaveFileDialog();

            FileDialog.Title = Properties.Resources.Caption_ExportAsXML;

            FileDialog.Filter = Properties.Resources.ExportAsXML_Filter;
            FileDialog.FilterIndex = 1;
            FileDialog.DefaultExt = ".xml";
            FileDialog.FileName = "";
            if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
            {
                FileDialog.Dispose();
                return;
            }

            FName = FileDialog.FileName;
            FileDialog.Dispose();
            this.Update();

            try
            {
                ShowStatus(Properties.Resources.Status_WritingFile, true);
                ucGantt1.gantt1.SerializeDataXML(FName);
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {
                ShowStatus();
                UpdateMenu();
            }
        }

        void mnuImportFromXML_Click(object sender, EventArgs e)
        {
            if (!AskSave())
                return;

            string FName;

            System.Windows.Forms.OpenFileDialog FileDialog = new System.Windows.Forms.OpenFileDialog();
            FileDialog.Title = Properties.Resources.Caption_ImportResourcePlannerXML;
            FileDialog.Filter = Properties.Resources.FileDialog_PPXML_Filter;
            FileDialog.CheckFileExists = true;
            FileDialog.Multiselect = false;
            FileDialog.DereferenceLinks = true;
            FileDialog.FilterIndex = 1;
            FileDialog.FileName = "";
            if (FileDialog.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
            {
                FileDialog.Dispose();
                return;
            }

            FName = FileDialog.FileName;
            FileDialog.Dispose();
            this.Update();

            try
            {
                ShowStatus(Properties.Resources.Status_ReadingFile, true);
                ucGantt1.gantt1.DeSerializeDataXML(FName, false);

                ucGantt1.gantt1.SetModified();
                ProjectFilename = "";
            }
            catch (Exception ex)
            {
                Program.CursorDefault();
                Program.ErrMsgBox(ex.Message);
            }
            finally
            {
                ShowStatus();
                UpdateMenu();
            }
        }

        void mnuPrintPageSetup_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Printing.PrintPageSetup();
        }

        void mnuPrintPreview_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Printing.PrintPreview();
        }

        void mnuPrint_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Printing.Print();
        }

        void mnuProjectSettings_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ShowProjectSettingsDialog();
            UpdateMenu();
        }

        void mnuProjectCalendars_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ShowCalendarListDialog();
            UpdateMenu();
        }

        void mnuAddResource_Click(object sender, EventArgs e)
        {
            Program.CursorWait();
            KS.Gantt.Dialogs.ResourceEditDialog Dlg = new KS.Gantt.Dialogs.ResourceEditDialog();
            Dlg.Edit(ucGantt1.gantt1, null);
            Dlg.ShowDialog(this);
            Dlg.Dispose();
            Dlg = null;
        }

        void mnuAddGroup_Click(object sender, EventArgs e)
        {
            Program.CursorWait();
            KS.Gantt.Dialogs.GroupEditDialog Dlg = new KS.Gantt.Dialogs.GroupEditDialog();
            Dlg.Edit(ucGantt1.gantt1, null);
            Dlg.ShowDialog(this);

            if (Dlg.OkPressed)
            {
                ucGantt1.gantt1.ViewMode = Gantt.ViewModes.Tasks;
                tabMain.SelectedTab = tabPlan;
            }

            Dlg.Dispose();
            Dlg = null;
        }

        void mnuAddTask_Click(object sender, EventArgs e)
        {
            Program.CursorWait();
            KS.Gantt.Dialogs.TaskEditDialog Dlg = new KS.Gantt.Dialogs.TaskEditDialog();
            Dlg.Edit(ucGantt1.gantt1, null);
            Dlg.ShowDialog(this);

            if (Dlg.OkPressed)
            {
                ucGantt1.gantt1.ViewMode = Gantt.ViewModes.Tasks;
                tabMain.SelectedTab = tabPlan;                
            }

            Dlg.Dispose();
            Dlg = null;
        }

        void mnuZoomIn_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ZoomIn();
        }

        void mnuZoomOut_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ZoomOut();
        }

        void mnuCenterNow_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.CenterNow();
        }

        void mnuZoomToFit_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ZoomToFit();
        }

        void mnuExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        void mnuTaskView_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ViewMode = KS.Gantt.Gantt.ViewModes.Tasks;
        }

        void mnuResourceView_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ViewMode = KS.Gantt.Gantt.ViewModes.Resources;
        }

        void mnuSelectAll_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.SelectAll();
        }

        void mnuInverseSelection_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.InverseSelection();
        }

        void mnuCollapseGroups_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.CollapseAll();
        }

        void mnuExpandGroups_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ExpandAll();
        }

        void mnuUndo_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Undo();
        }

        void mnuRedo_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Redo();
        }

        void mnuCut_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Cut();
        }

        void mnuCopy_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Copy();
        }

        void mnuPaste_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Paste();
        }

        void mnuDelete_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.Delete();
        }

        void gantt1_BeforeDelete(object sender, KS.Gantt.GanttCancelEventArgs e)
        {
            string strMessage = "";

            switch (ucGantt1.gantt1.ViewMode)
            {
                case Gantt.ViewModes.Tasks:
                    strMessage = String.Format(Properties.Resources.Question_DeleteItems, ucGantt1.gantt1.SelectedItems.Count.ToString());
                    break;

                case Gantt.ViewModes.Resources:
                    strMessage = String.Format(Properties.Resources.Question_DeleteItems, ucGantt1.gantt1.SelectedResources.Count.ToString());
                    break;
            }

            if (Program.QuestionMsgBox(strMessage) != DialogResult.Yes)
                e.Cancel = true;
        }

        void gantt1_ItemDoubleClick(object sender, KS.Gantt.GanttItemMouseEventArgs e)
        {
            // Don't allow editing of Groups ?
            // Uncomment this
            //if (e.Item.ItemType != GanttItemType.GroupItem)
            
            ucGantt1.gantt1.EditItem(e.Item);
        }

        void gantt1_ItemsChanged(object sender, EventArgs e)
        {
            UpdateMenu();
        }

        void gantt1_SelectionChanged(object sender)
        {
            UpdateMenu();
        }

        void clipChange_ClipboardChanged(object sender, EventArgs e)
        {            
            Application.DoEvents();
            UpdateMenu();
        }

        void ganttDataGrid1_UpdateEditMenu(object sender, EventArgs e)
        {
            UpdateMenu();
        }        

        private void UpdateMenu()
        {            
            // Resourceplanner specific
            mnuAddTask.Enabled = ucGantt1.gantt1.Resources.Count > 0;
            mnuAddTask1.Enabled = ucGantt1.gantt1.Resources.Count > 0;

            mnuMoveUp.Enabled = ucGantt1.ganttDataGrid1.CanMoveRowUp;
            mnuMoveUp1.Enabled = ucGantt1.ganttDataGrid1.CanMoveRowUp;

            mnuMoveDown.Enabled = ucGantt1.ganttDataGrid1.CanMoveRowDown;
            mnuMoveDown1.Enabled = ucGantt1.ganttDataGrid1.CanMoveRowDown;

            // Other            

            mnuUndo.Enabled = ucGantt1.ganttDataGrid1.CanUndo;
            mnuUndo1.Enabled = mnuUndo.Enabled;
            mnuRedo.Enabled = ucGantt1.ganttDataGrid1.CanRedo;
            mnuRedo1.Enabled = mnuRedo.Enabled;            

            mnuSave.Enabled = ucGantt1.gantt1.Modified;
            mnuSave1.Enabled = mnuSave.Enabled;

            mnuSaveAs.Enabled = ProjectFilename != "";

            mnuPrint.Enabled = true;            

            bool bGanttVisible = tabMain.SelectedIndex < 1;

            mnuCut.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanCut;            
            mnuCopy.Enabled = bGanttVisible & ucGantt1.ganttDataGrid1.CanCopy;            
            mnuPaste.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanPaste;            
            mnuDelete.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanDelete;
            mnuSelectAll.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanSelectAll;
            mnuInverseSelection.Enabled = bGanttVisible && ucGantt1.ganttDataGrid1.CanInverseSelection;            

            mnuZoomIn.Enabled = bGanttVisible;
            mnuZoomOut.Enabled = bGanttVisible;
            mnuZoomToFit.Enabled = bGanttVisible;
            mnuCenterNow.Enabled = bGanttVisible;

            mnuCollapseGroups.Enabled = bGanttVisible;
            mnuExpandGroups.Enabled = bGanttVisible;

            mnuFind.Enabled = bGanttVisible && ucGantt1.gantt1.CanSearch;
            mnuFind1.Enabled = mnuFind.Enabled;
            mnuFindMore.Enabled = bGanttVisible && ucGantt1.gantt1.CanSearchMore;
            mnuFindMore1.Enabled = mnuFindMore.Enabled;

            bool bTaskviewVisible = tabMain.SelectedIndex < 1;            
            
            if (ucGantt1.gantt1.MaxZoomLevel == Gantt.ZoomLevels.Granularity)
            {
                mnuMaxZoomLevelGranularity.Checked = true;
                mnuMaxZoomLevelMinute.Checked = false;
            }
            else
            {
                mnuMaxZoomLevelGranularity.Checked = false;
                mnuMaxZoomLevelMinute.Checked = true;
            }                        

            string strText = ucGantt1.gantt1.Project.Text;
            if (ProjectFilename != "")
                strText += " [" + KS.Common.Strings.GetFilename(ProjectFilename) + "]";            

            strText += " - " + Program.DialogTitle;
            this.Text = strText;
        }

        void gantt1_ItemHover(object sender, GanttItemEventArgs e)
        {
            ShowItemStatus(e.Item);
        }

        void gantt1_ItemsResized(object sender, EventArgs e)
        {
            ShowItemStatus(null);
        }

        void gantt1_ItemsMoved(object sender, EventArgs e)
        {
            ShowItemStatus(null);
        }

        void gantt1_MovingItem(object sender, GanttItemEventArgs e)
        {
            ShowItemStatus(e.Item);
        }

        private bool LastEnableGUI = true;
        public void EnableGUI(bool bEnable)
        {
            if (LastEnableGUI == bEnable)
                return;

            LastEnableGUI = bEnable;

            foreach (Control c in this.Controls)
            {
                c.Enabled = bEnable;
            }

            this.toolStripStatusLabelTaskName.Enabled = true;
            this.toolStripStatusLabelStatus.Enabled = true;
            this.Focus();
        }

        public void ShowStatus()
        {
            ShowStatus("", false);
        }

        public void ShowStatus(string Status, bool bWaitCursor)
        {
            if (Status == "")
            {
                Status = Properties.Resources.StatusReady;
                Program.CursorDefault();
                toolStripStatusLabelStatus.Image = null;
            }
            else
            {
                if (bWaitCursor)
                {
                    Program.CursorWait();
                    toolStripStatusLabelStatus.Image = Properties.Resources.wait;
                }
            }

            toolStripStatusLabelStatus.Text = Status;
            statusStrip1.Update();
        }

        private string LastItemStatusKey = null;
        private void ShowItemStatus(GanttItem item)
        {
            if (!(ucGantt1.gantt1.IsBarMoving || ucGantt1.gantt1.IsBarSizing))
            {
                if ((LastItemStatusKey == null && item == null) || (LastItemStatusKey != null && item != null && item.Key == LastItemStatusKey))
                    return;
            }

            MethodInvoker ShowItemStatusInvoker = delegate
            {
                if (item == null)
                {
                    toolStripStatusLabelTaskName.Text = "";
                    toolStripStatusLabelStartDate.Text = "";
                    toolStripStatusLabelEndDate.Text = "";
                    toolStripStatusLabelDuration.Text = "";
                    LastItemStatusKey = null;
                }
                else
                {
                    toolStripStatusLabelTaskName.Text = item.Text;
                    toolStripStatusLabelStartDate.Text = item.StartDateForDisplay;
                    toolStripStatusLabelEndDate.Text = item.EndDateForDisplay;
                    toolStripStatusLabelDuration.Text = item.DurationForDisplay;
                    LastItemStatusKey = item.Key;
                }
            };

            Invoke(ShowItemStatusInvoker);
        }

        // *** MRUManager ***
        public void OpenMRUFile(string fileName)
        {
            if (!AskSave())
                return;

            OpenFile(fileName);
        }

        void mnuFind_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.ShowSearchDialog();
            UpdateMenu();
        }

        void mnuFindMore_Click(object sender, EventArgs e)
        {
            ucGantt1.gantt1.Search();
        }

        private void SetUpContextMenu()
        {
            if (m_ContextMenu != null)
            {
                m_ContextMenu.Dispose();
                m_ContextMenu = null;
            }

            m_ContextMenu = new ContextMenuStrip();

            ToolStripItem cmEdit = m_ContextMenu.Items.Add(mnuEdit.Text + "..");
            cmEdit.Font = new Font(mnuEdit.Font.FontFamily, mnuEdit.Font.Size, FontStyle.Bold);
            cmEdit.Click += new EventHandler(cmEdit_Click);
            m_ContextMenu.Items.Add(new ToolStripSeparator());

            foreach (ToolStripItem item in mnuEdit.DropDownItems)
            {
                if (item is ToolStripSeparator)
                {
                    m_ContextMenu.Items.Add(new ToolStripSeparator());
                }
                else
                {
                    ToolStripItem ti = m_ContextMenu.Items.Add(item.Text, item.Image);
                    ti.Name = item.Name;                    
                    MethodAndSource.CopyEvents(item, ti, "Click");
                }
            }

            ucGantt1.gantt1.ContextMenuStrip = m_ContextMenu;
            m_ContextMenu.Opening += new CancelEventHandler(m_ContextMenu_Opening);
        }

        void cmEdit_Click(object sender, EventArgs e)
        {
            if (ucGantt1.gantt1.LastSelectedItem != null)
                ucGantt1.gantt1.EditItem(ucGantt1.gantt1.LastSelectedItem);
        }

        void m_ContextMenu_Opening(object sender, CancelEventArgs e)
        {
            m_ContextMenu.Items[0].Visible = ucGantt1.gantt1.LastSelectedItem != null;
            m_ContextMenu.Items[1].Visible = ucGantt1.gantt1.LastSelectedItem != null;

            m_ContextMenu.Items["mnuUndo"].Enabled = ucGantt1.gantt1.CanUndo;
            m_ContextMenu.Items["mnuRedo"].Enabled = ucGantt1.gantt1.CanRedo;

            m_ContextMenu.Items["mnuCut"].Enabled = ucGantt1.gantt1.CanCut;
            m_ContextMenu.Items["mnuDelete"].Enabled = ucGantt1.gantt1.CanDelete;
            m_ContextMenu.Items["mnuCopy"].Enabled = ucGantt1.gantt1.CanCopy;
            m_ContextMenu.Items["mnuPaste"].Enabled = ucGantt1.gantt1.CanPaste;

            m_ContextMenu.Items["mnuSelectAll"].Enabled = ucGantt1.gantt1.CanSelectAll;
            m_ContextMenu.Items["mnuInverseSelection"].Enabled = ucGantt1.gantt1.CanSelectAll;

            m_ContextMenu.Items["mnuFind"].Enabled = ucGantt1.gantt1.CanSearch;
            m_ContextMenu.Items["mnuFindMore"].Enabled = ucGantt1.gantt1.CanSearchMore;

            m_ContextMenu.Items["mnuMoveUp"].Enabled = ucGantt1.ganttDataGrid1.CanMoveRowUp;
            m_ContextMenu.Items["mnuMoveDown"].Enabled = ucGantt1.ganttDataGrid1.CanMoveRowDown;            
        }


        // *** Move Up/Down ***
        
        void ganttDataGrid1_GotFocus(object sender, EventArgs e)
        {
            UpdateMenu();
        }        

        void ganttDataGrid1_LostFocus(object sender, EventArgs e)
        {
            UpdateMenu();
        }

        void ganttDataGrid1_SelectedRowIndexChanged(object sender, EventArgs e)
        {            
            UpdateMenu();
        }        
        
        void mnuMoveUp_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.MoveRowUp();
            UpdateMenu();
        }

        void mnuMoveDown_Click(object sender, EventArgs e)
        {
            ucGantt1.ganttDataGrid1.MoveRowDown();
            UpdateMenu();
        }        
    }    
}